package de.duehl.pentris.logic;

public class Coordinate {

    /** Zeile beginnend bei 0 (unten). */
    private int row;

    /** Spalte beginnend bei 0 (links). */
    private int col;

    /**
     * Konstruktor.
     *
     * @param row
     *            Zeile beginnend bei 0 (unten).
     * @param col
     *            Spalte beginnend bei 0 (links).
     */
    public Coordinate(int row, int col) {
        this.row = row;
        this.col = col;
    }

    /**
     * Konstruktor
     *
     * @param that
     *            Andere Koordinate, deren Werte kopiert werden.
     */
    public Coordinate(Coordinate that) {
        this.row = that.row;
        this.col = that.col;
    }

    /** Getter für die Zeile beginnend bei 0 (unten). */
    public int getRow() {
        return row;
    }

    /** Getter für die Spalte beginnend bei 0 (links). */
    public int getCol() {
        return col;
    }

    /**
     * In StoneFactory werden die Steine mit rows 0, 1, ... angelegt. Hier
     * werden diese nun ans obere Ende korrigiert.
     */
    public void adjustCoordinates() {
        row = Constants.ROWS-1 - row;
    }

    @Override
    public String toString() {
        return "Coordinate [row=" + row + ", col=" + col + "]";
    }

    /** Bewegt die Koordinate eine Spalte nach links. */
    public void left() {
        --col;
    }

    /** Bewegt die Koordinate eine Spalte nach rechts. */
    public void right() {
        ++col;
    }

    /** Bewegt die Koordinate eine Zeile nach unten. */
    public void drop() {
        --row;
    }

    /** Bewegt die Koordinate eine Zeile nach oben. */
    public void up() {
        ++row;
    }

    /**
     * Überprüft, ob die Koordinate im zulässigen Bereich liegt.                <br/><br/>
     *
     * Kriterien für den zulässigen Bereich:
     *     0 <= row < Constants.ROWS und
     *     0 <= col < Constants.COLS
     *
     * @return Wahrheitswert
     */
    public boolean isValid() {
        if (row >= Constants.ROWS) {
            return false;
        }
        if (row < 0) {
            return false;
        }
        if (col >= Constants.COLS) {
            return false;
        }
        if (col < 0) {
            return false;
        }
        return true;
    }

}
