#!/usr/bin/perl
#------------------------------------------------------------------------------
# program     : p e n t r i s . p l
#------------------------------------------------------------------------------
# author      : Christian Duehl
# created     : 2004-08-20
# last change : 2004-08-25
# task        : Pentris game (Tetris clone)
# parameter   : none
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# packages / pragmas:
#------------------------------------------------------------------------------
use strict;
use warnings;
use Tk;
use Tk::LabFrame;
use Tk::ErrorDialog;
#use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;

package main;

my %Config = (
               version      => '0.0.4',
               version_date => '2004-08-25',
               Menuefont    => '{Arial} 8 {normal}',
               highscore_p  => 'highscore.pentris',
               highscore_t  => 'highscore.tetris',
             );

my %C = (
          rows       => 24, # 20, # 26 (FZEIL)
          cols       => 13, # 10, # 16 (FSPALT)
          rowpix     => 31, # 22, # 16 (ZPIX)
          colpix     => 31, # 20, # 14 (SPIX)
          withgrid   => 1,    # 1 or 0 for showing the grid or not
          points     => 0,
          startspeed => 1000, # milliseconds between stone falls
          speed      => 0,
          gametype   => 'Pentris',
          askforexit => 1,
        );

        # further fields filled in %C during the program:
        #
        # mw
        # canvas
        # stone
        # field
        # width
        # height
        # canvaswidth
        # canvasheigh
        # stone
        # field
        # objects
        # width
        # height
        # canvaswidth
        # canvasheightt
        # after

my %Stones;
{
my $Middle = int( $C{cols} / 2 );#-1;
   %Stones = ('Tetris langer gerader Vierer' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 1,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [0, $Middle],
                                  [0, $Middle-1],
                                  [0, $Middle+1],
                                  [0, $Middle+2],
                                ],
                  },
              'Tetris L mit Nase nach rechts' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 2,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [1, $Middle],
                                  [0, $Middle+1],
                                  [0, $Middle],
                                  [2, $Middle],
                                ],
                  },
              'Tetris L mit Nase nach links' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 3,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [1, $Middle],
                                  [0, $Middle-1],
                                  [0, $Middle],
                                  [2, $Middle],
                                ],
                  },
              'Tetris Winkel nach rechts' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 4,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [1, $Middle+1],
                                  [2, $Middle+1],
                                ],
                  },
              'Tetris Winkel nach links' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 5,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [1, $Middle+1],
                                  [0, $Middle+1],
                                  [1, $Middle],
                                  [2, $Middle],
                                ],
                  },
              'Tetris Quadrat' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 6,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [0, $Middle],
                                  [0, $Middle+1],
                                  [1, $Middle],
                                  [1, $Middle+1],
                                ],
                  },
              'Tetris Mittelnase' =>
                  {
                      weight => 6,  # factor for random choice
                      color  => 7,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Tetris',
                      coords => [ [0, $Middle],
                                  [0, $Middle-1],
                                  [0, $Middle+1],
                                  [1, $Middle],
                                ],
                  },
              '1er Quadrat' =>
                  {
                      weight => 3,  # factor for random choice
                      color  => 8,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Extra',
                      coords => [ [0, $Middle],
                                ],
                  },
              '2er gerade' => # wie auch sonst?
                  {
                      weight => 3,  # factor for random choice
                      color  => 9,  # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Extra',
                      coords => [ [0, $Middle],
                                  [1, $Middle],
                                ],
                  },
              '3er gerade' => # wie auch sonst?
                  {
                      weight => 3,  # factor for random choice
                      color  => 10, # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Extra',
                      coords => [ [0, $Middle],
                                  [1, $Middle],
                                  [2, $Middle],
                                ],
                  },
              '3er Winkel' => # wie auch sonst?
                  {
                      weight => 3,  # factor for random choice
                      color  => 11, # stone color (SF)
                      bonus  => 0,  # bonus points (SPB)
                      type   => 'Extra',
                      coords => [ [0, $Middle+1],
                                  [1, $Middle+1],
                                  [0, $Middle],
                                ],
                  },
              'Pentomino I' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 12, # stone color (SF)
                      bonus  => 10, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [0, $Middle],
                                  [0, $Middle-1],
                                  [0, $Middle+1],
                                  [0, $Middle+2],
                                  [0, $Middle-2],
                                ],
                  },
              'Pentomino L Nase nach oben rechts' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 13, # stone color (SF)
                      bonus  => 10, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [0, $Middle+1],
                                  [2, $Middle],
                                  [3, $Middle],
                                ],
                  },
              'Pentomino L Nase nach oben links' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 14, # stone color (SF)
                      bonus  => 10, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle+1],
                                  [0, $Middle+1],
                                  [0, $Middle],
                                  [2, $Middle+1],
                                  [3, $Middle+1],
                                ],
                  },
              'Pentomino Y Nase nach oben rechts' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 15, # stone color (SF)
                      bonus  => 20, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle],
                                  [3, $Middle],
                                  [1, $Middle+1],
                                ],
                  },
              'Pentomino Y Nase nach oben links' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 16, # stone color (SF)
                      bonus  => 20, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle+1],
                                  [0, $Middle+1],
                                  [2, $Middle+1],
                                  [3, $Middle+1],
                                  [1, $Middle],
                                ],
                  },
              'Pentomino V' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 17, # stone color (SF)
                      bonus  => 20, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [0, $Middle-1],
                                  [0, $Middle],
                                  [0, $Middle+1],
                                  [1, $Middle-1],
                                  [2, $Middle-1],
                                ],
                  },
              'Pentomino T' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 18, # stone color (SF)
                      bonus  => 15, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [0, $Middle],
                                  [0, $Middle+1],
                                  [0, $Middle-1],
                                  [1, $Middle],
                                  [2, $Middle],
                                ],
                  },
              'Pentomino X' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 19, # stone color (SF)
                      bonus  => 25, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle],
                                  [1, $Middle+1],
                                  [1, $Middle-1],
                                ],
                  },
              'Pentomino P Bauch nach rechts' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 20, # stone color (SF)
                      bonus  => 15, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle],
                                  [1, $Middle+1],
                                  [0, $Middle+1],
                                ],
                  },
              'Pentomino P Bauch nach links' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 21, # stone color (SF)
                      bonus  => 15, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle+1],
                                  [0, $Middle+1],
                                  [2, $Middle+1],
                                  [1, $Middle],
                                  [0, $Middle],
                                ],
                  },
              'Pentomino F richtigherum' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 22, # stone color (SF)
                      bonus  => 30, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle],
                                  [1, $Middle-1],
                                  [0, $Middle+1],
                                ],
                  },
              'Pentomino F gespiegelt' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 23, # stone color (SF)
                      bonus  => 30, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle],
                                  [1, $Middle+1],
                                  [0, $Middle-1],
                                ],
                  },
              'Pentomino U' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 24, # stone color (SF)
                      bonus  => 20, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [0, $Middle],
                                  [0, $Middle-1],
                                  [1, $Middle-1],
                                  [1, $Middle+1],
                                  [0, $Middle+1],
                                ],
                  },
              'Pentomino Z oben rechts' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 25, # stone color (SF)
                      bonus  => 30, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [1, $Middle-1],
                                  [1, $Middle+1],
                                  [0, $Middle-1],
                                  [2, $Middle+1],
                                ],
                  },
              'Pentomino Z oben links' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 26, # stone color (SF)
                      bonus  => 30, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [1, $Middle-1],
                                  [1, $Middle+1],
                                  [2, $Middle-1],
                                  [0, $Middle+1],
                                ],
                  },
              'Pentomino W' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 27, # stone color (SF)
                      bonus  => 30, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [0, $Middle],
                                  [0, $Middle-1],
                                  [1, $Middle+1],
                                  [2, $Middle+1],
                                ],
                  },
              'Pentomino N von links oben nach rechts unten' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 28, # stone color (SF)
                      bonus  => 15, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle+1],
                                  [1, $Middle],
                                  [0, $Middle],
                                  [2, $Middle+1],
                                  [3, $Middle+1],
                                ],
                  },
              'Pentomino N von rechts oben nach links unten' =>
                  {
                      weight => 1,  # factor for random choice
                      color  => 29, # stone color (SF)
                      bonus  => 15, # bonus points (SPB)
                      type   => 'Pentris',
                      coords => [ [1, $Middle],
                                  [1, $Middle+1],
                                  [0, $Middle+1],
                                  [2, $Middle],
                                  [3, $Middle],
                                ],
                  },
             );
}

my @Color = ('#000000', # black = background, don't use this!

             '#EE9900', # Tetris I          1
             '#FF0033', # Tetris L r        2
             '#2200FF', # Tetris L l        3
             '#22CCCC', # Tetris N l-r      4
             '#CC22CC', # Tetris N r-l      5
             '#EEDD00', # Tetris square     6
             '#00BB00', # Tetris T          7

             '#883300', # 1er               8
             '#AA5500', # 2er               9
             '#CC7700', # 3er gerade       10
             '#660066', # 3er Winkel       11

             '#FFAA11', # Pentris I        12
             '#CC0011', # Pentris L l      13
             '#1100CC', # Pentris L r      14
             '#00AA44', # Pentris Y r      15   x
             '#AA5577', # Pentris Y l      16
             '#990099', # Pentris V        17
             '#009900', # Pentris T        18
             '#006600', # Pentris X        19
             '#1100AA', # Pentris P r      20
             '#AA0011', # Pentris P l      21
             '#CCCCFF', # Pentris F right  22
             '#CCFFCC', # Pentris F wrong  23
             '#88EEAA', # Pentris U        24   x
             '#FF00FF', # Pentris Z r      25
             '#00FF00', # Pentris Z l      26
             '#110099', # Pentris W        27
             '#11DDAA', # Pentris N l-r    28
             '#DD88AA', # Pentris N r-l    29
            );


$C{withgrid} = 0 if grep {$_ eq 'nogrid'} @ARGV;
runtk();
exit;


package Pentris::Base;
    #--------------------------------------------------------------------------
    # Class          : P e n t r i s : : B a s e
    # derived from   : none
    #--------------------------------------------------------------------------
    # Author         : Christian Duehl
    # Task           : Base class for this project.
    # Attributes     : none, no objects can be created
    # Class uethods  : _property
    # Object methods : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created.
    #--------------------------------------------------------------------------

sub _property {
    #--------------------------------------------------------------------------
    # Method       : _ p r o p e r t y
    # Class        : Pentris::Base
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Class method for easy Getter/Setter.
    # Parameter    : 1) Attribute
    #                2) Value (optional)
    # Return value : the value (getter) or the old value (setter)
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-13 - CD - Created
    #--------------------------------------------------------------------------

    my ($self, $attr, $value) = @_;

    die "no attribute given in _property" unless defined $attr;

    return $self->{$attr} if @_ != 3;

    my $old_value  = $self->{$attr};
    $self->{$attr} = $value;
    return $old_value;

} # sub Pentris::Base::_property


package Pentris::Stone;
    #--------------------------------------------------------------------------
    # Class          : P e n t r i s : : B a s e
    # derived from   : Pentris::Base
    #--------------------------------------------------------------------------
    # Author         : Christian Duehl
    # Task           : Stone class
    # Attributes     : color  : Color id (for @Color)
    #                  type   : Stone type (Pentris, Tetris or other)
    #                  coords : Reference to an AoA with the coordinates -
    #                           like [ [row0, col9], [row1, col1] ... ]
    #                  bonus  : Bonus points for dropping this stone into the
    #                           grid.
    # Class methods  : new    ; Constructor
    # Object methods : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created.
    #--------------------------------------------------------------------------


use base qw/Pentris::Base/;


sub new {
    #--------------------------------------------------------------------------
    # Method       : n e w
    # Class        : Pentris::Stone
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Class method : Constructor
    # Parameter    : 1) Type of the stone
    # Return value : the object
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    #--------------------------------------------------------------------------

    my ($ref, $type) = (@_, 'random');
    my $class = (ref $ref) || $ref;

    my $self  = {};

    bless $self, $class;

    #--------------------------------------------------------------------------
    # Random choice:
    #--------------------------------------------------------------------------
    if ($type eq 'random') {
        my @choice;
        for my $stone (keys %Stones) {
            if ($Stones{$stone}->{type} eq 'Tetris' or
                $C{gametype} ne 'Tetris') {
                push @choice, $stone for 0 .. $Stones{$stone}->{weight};
            }
        }
        $type = $choice[ int rand scalar @choice ];
        die "Undefined type" unless defined $type;
    }

    #--------------------------------------------------------------------------
    # Init stone:
    #--------------------------------------------------------------------------
    die "stone '$type' is not known" unless exists $Stones{$type};
    $self->color ($Stones{$type}->{color});
    $self->type  ($type                  );
    my @coords;
    for my $square (@{ $Stones{$type}->{coords} }) {
        my ($ri, $ci) = @$square;
        push @coords, [ $ri, $ci ];
    }
    $self->coords([@coords]);
    $self->bonus ($Stones{$type}->{bonus});


    return $self;

} # sub RDW06::Field::new


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub color     { return shift->_property('color',  @_) }
sub type      { return shift->_property('type',   @_) }
sub coords    { return shift->_property('coords', @_) }
sub bonus     { return shift->_property('bonus',  @_) }



package main;


sub runtk  {
    #--------------------------------------------------------------------------
    # Function     : r u n t k
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Starts the Tk gui and the game
    # Parameter    : none
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    # 0.0.2 - 2004-08-21 - CD - Quit-Button, ESC-Binding
    # 0.0.3 - 2004-08-24 - CD - Catch the close-window-button
    #                           Making the grid optional.
    # 0.0.4 - 2004-08-24 - CD - Toggle Grid
    # 0.0.5 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # MainWindow:
    #--------------------------------------------------------------------------
    $C{mw} = new MainWindow;
    $C{mw}->title('Pentris');

    #--------------------------------------------------------------------------
    # Show the game field:
    #--------------------------------------------------------------------------
    my $gf = $C{mw}->LabFrame(-label     => 'Spielfeld',
                              -labelside => 'acrosstop',
                             )
                       ->pack(-side      => 'top',
                              -expand    => 1,
                              -fill      => 'both',
                             );

    $C{canvaswidth}  = ($C{withgrid}+$C{colpix}) * $C{cols} - $C{withgrid};
    $C{canvasheight} = ($C{withgrid}+$C{rowpix}) * $C{rows} - $C{withgrid};
    $C{width}        = $C{canvaswidth}  + 2;  # why a ever, but it is necessary
    $C{height}       = $C{canvasheight} + 2;

    $C{canvas} = $gf->Canvas(-background  => 'black',
                             -width       => $C{canvaswidth},
                             -height      => $C{canvasheight},
                            )
                      ->pack(-side        => 'top',
                             -expand      => 0,
                             -fill        => 'none',
                            );

    draw_grid();

    #--------------------------------------------------------------------------
    # Game informations:
    #--------------------------------------------------------------------------
    {
        #----------------------------------------------------------------------
        # Game type
        #----------------------------------------------------------------------
        my $fi = $C{mw}->Frame()
                        ->pack(-side      => 'top',
                               -expand    => 1,
                               -fill      => 'both',
                              );
        my $pf = $fi->LabFrame(-label     => 'Spieltyp',
                               -labelside => 'acrosstop',
                              )
                        ->pack(-side      => 'left',
                               -expand    => 1,
                               -fill      => 'both',
                              );
        $pf->Label(-textvariable => \$C{gametype})
            ->pack(-side         => 'top',
                   -expand       => 0,
                   -fill         => 'none',
                  );

        #----------------------------------------------------------------------
        # Points
        #----------------------------------------------------------------------
        my $pp = $fi->LabFrame(-label     => 'Punkte',
                               -labelside => 'acrosstop',
                              )
                        ->pack(-side      => 'left',
                               -expand    => 1,
                               -fill      => 'both',
                              );
        $pp->Label(-textvariable => \$C{points})
            ->pack(-side         => 'top',
                   -expand       => 0,
                   -fill         => 'none',
                  );
    }

    #--------------------------------------------------------------------------
    # Quit-Button:
    #--------------------------------------------------------------------------
    my $quit_b = $C{mw}->Button(-text => 'Ende',
                                -command => [ \&my_exit, $C{mw} ],
                               )
                         ->pack(-side    => 'top',
                                -padx    => 10,
                                -pady    => 2,
                                -expand  => 1,
                                -fill    => 'x',
                               );

    #--------------------------------------------------------------------------
    # Field / Stone not filled:
    #--------------------------------------------------------------------------
    $C{field} = 'nofield';
    $C{stone} = 'nostone';

    #--------------------------------------------------------------------------
    # Menu:
    #--------------------------------------------------------------------------
    # 1. create menubar:
    my $menu = $C{mw}->Menu(-type => 'menubar',
                            -font => $Config{Menuefont},
                           );

    # 2. tell main window, that this is his menu:
    $C{mw}->configure(-menu => $menu);

    # 3. Add menu entries:
    $menu->cascade(-label     => 'Spiel',
                   -underline => 0,
                   -font      => $Config{Menuefont},
                  );
    $menu->separator(); # the rest of the menues is at the right side
    $menu->cascade(-label     => 'Hilfe',
                   -underline => 0,
                   -font      => $Config{Menuefont},
                  );

    # 4. Define the sub menues:
    # First the game menu:
    my $game_menu = $menu
         ->Menu(-font      => $Config{Menuefont},
                -tearoff   => 0,
                -menuitems => [
                                [
                                 'command'    => 'Neues Spiel',
                                 -command     => \&start_game,
                                 -accelerator => 'F2',
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                                [
                                 'command'    => 'Toggle Grid',
                                 -command     => \&toggle_grid,
                                 -accelerator => 'F3',
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                                [
                                 'command'    => 'Show highscores',
                                 -command     => \&show_highscore,
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                                '-', # seperator
                                [
                                 'command'    => 'Beenden',
                                 -command     => \&my_exit,
                                 -accelerator => 'ESC',
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                              ],
               );
    # Tell the menubar, that this submenu is his game menu:
    $menu->entryconfigure('Spiel', -menu => $game_menu);

    # Second the help menu:
    my $help_menu = $menu
         ->Menu(-font      => $Config{Menuefont},
                -tearoff   => 0,
                -menuitems => [
                                [
                                 'command'    => "Hilfe",
                                 -command     => [ \&help, $C{mw} ],
                                 -accelerator => 'F1',
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                                [
                                 'command'    => "Historie",
                                 -command     => [ \&history, $C{mw} ],
                                 -font        => $Config{Menuefont},
                                 -underline   => 1,
                                ],
                                '-',
                                [
                                 'command'    => "Ueber Pentris",
                                 -command     => [ \&over_pentris, $C{mw} ],
                                 -font        => $Config{Menuefont},
                                 -underline   => 0,
                                ],
                              ],
               );
    $menu->entryconfigure('Hilfe', -menu => $help_menu);

    #--------------------------------------------------------------------------
    # Bindings:
    #--------------------------------------------------------------------------
    $C{mw}->bind('<Alt-s>',  sub {$menu->postcascade('Spiel')});
    $C{mw}->bind('<Alt-h>',  sub {$menu->postcascade('Hilfe')});

    $C{mw}->bind('<Escape>', sub {$quit_b->invoke()});
    $C{mw}->bind('<F1>',     \&help                 );
    $C{mw}->bind('<F2>',     \&start_game           );
    $C{mw}->bind('<F3>',     \&toggle_grid          );

    #--------------------------------------------------------------------------
    # Catch the close-window-button:
    #--------------------------------------------------------------------------
    $C{mw}->protocol("WM_DELETE_WINDOW" => sub { $quit_b->invoke() });

    Tk::MainLoop();
} # sub runtk


sub draw_grid {
    #--------------------------------------------------------------------------
    # Function     : d r a w _ g r i d
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Draw the grid into the given canvas.
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Making the grid optional.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my $gridcolor      = '#FFFFFF';

    if ($C{withgrid}) {
        $C{canvas}->createLine(($C{withgrid} + $C{colpix}) * $_ + $C{withgrid},
                         1      ,
                         ($C{withgrid} + $C{colpix}) * $_ + $C{withgrid},
                         $C{height},
                         -fill => $gridcolor)
                         for 0 .. $C{cols}-1;
        $C{canvas}->createLine(1,
                         ($C{withgrid} + $C{rowpix}) * $_ + $C{withgrid},
                         $C{width},
                         ($C{withgrid} + $C{rowpix}) * $_ + $C{withgrid},
                         -fill => $gridcolor)
                         for 0 .. $C{rows}-1;
        # painting the right (lower) line of the grid for each $_
    }

} # sub draw_grid


sub start_game {
    #--------------------------------------------------------------------------
    # Function     : s t a r t _ g a m e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Starts the game.
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    # 0.0.2 - 2004-08-22 - CD - Dialog and much more added.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # Cancel drops
    #--------------------------------------------------------------------------
    $C{after}->cancel() if exists $C{after} and defined $C{after};

    #--------------------------------------------------------------------------
    # Delete all objects on the canvas:
    #--------------------------------------------------------------------------
    $C{canvas}->delete('all');
    $C{objects} = [];

    #--------------------------------------------------------------------------
    # Reset game Variables:
    #--------------------------------------------------------------------------
    $C{points} = 0;
    $C{speed}  = $C{startspeed};

    #--------------------------------------------------------------------------
    # Draw the grid:
    #--------------------------------------------------------------------------
    draw_grid();

    #--------------------------------------------------------------------------
    # Ask the user for Pentris or Tetris:
    #--------------------------------------------------------------------------
    {
        #my $icon = $C{mw}->Photo(-file   => 'xyz.bmp',
        #                      -height => 32,
        #                      -width  => 32,
        #                     );
        my $dialog = $C{mw}->Dialog(-text           => 'Welche Spielart?',
                                    -bitmap         => 'question',
                                   #-image          => $icon,
                                    -title          => 'Welche Spielart?',
                                    -default_button => 'Pentris',
                                    -buttons        => [ 'Tetris', 'Pentris' ],
                                   #-font           => '{Courier New} 10 {normal}',
                                    -font           => '{Times New Roman} 12 {normal}',
                                   );
        $C{gametype} = $dialog->Show(-global);
    }

    #--------------------------------------------------------------------------
    # Create the new field and the first stone:
    #--------------------------------------------------------------------------
    $C{field} = [];
    for my $r (0..$C{rows}-1) {
        for my $c (0..$C{cols}-1) {
            $C{field}->[$r][$c] = 0;
        }
    }

    #show_field($C{field});
    $C{stone}   = new Pentris::Stone;
    $C{objects} = [ show_stone() ];

    #--------------------------------------------------------------------------
    # Bindings:
    #--------------------------------------------------------------------------
    $C{mw}->bind('<Key-p>',     \&pause_game             );
    $C{mw}->bind('<Pause>',     \&pause_game             );
    $C{mw}->bind('<Right>',     [ \&move_stone, 'right' ]);
    $C{mw}->bind('<Left>',      [ \&move_stone, 'left'  ]);
    $C{mw}->bind('<Down>',      [ \&move_stone, 'down'  ]);
    $C{mw}->bind('<Key-space>', \&drop_stone             );
    $C{mw}->bind('<Up>',        [ \&move_stone, 'turn'  ]);

    #--------------------------------------------------------------------------
    # Let the stone fall down in one second:
    #--------------------------------------------------------------------------
    $C{after} = $C{mw}->after($C{speed}, \&fall_down);

} # sub start_game


sub toggle_grid {
    #--------------------------------------------------------------------------
    # Function     : t o g g l e _ g r i d
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : toggles the grid
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-24 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    if ($C{field} ne 'nofield' and $C{stone} ne 'nostone') {
        #----------------------------------------------------------------------
        # Cancel drops
        #----------------------------------------------------------------------
        $C{after}->cancel() if exists $C{after} and defined $C{after};

        #----------------------------------------------------------------------
        # Release Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Right>',    '');
        $C{mw}->bind('<Left>',     '');
        $C{mw}->bind('<Down>',     '');
        $C{mw}->bind('<Spacebar>', '');
        $C{mw}->bind('<Up>',       '');
    }

    #--------------------------------------------------------------------------
    # Delete all objects on the canvas:
    #--------------------------------------------------------------------------
    $C{canvas}->delete('all');
    $C{objects} = [];

    #--------------------------------------------------------------------------
    # Toggle the grid:
    #--------------------------------------------------------------------------
    $C{withgrid}     ^= 1;
    $C{canvaswidth}   = ($C{withgrid}+$C{colpix}) * $C{cols} - $C{withgrid};
    $C{canvasheight}  = ($C{withgrid}+$C{rowpix}) * $C{rows} - $C{withgrid};
    $C{width}         = $C{canvaswidth}  + 2;  # why a ever, but it is necessary
    $C{height}        = $C{canvasheight} + 2;
    $C{canvas}->configure(-width       => $C{canvaswidth},
                          -height      => $C{canvasheight});

    #--------------------------------------------------------------------------
    # Draw the grid:
    #--------------------------------------------------------------------------
    draw_grid();

    if ($C{field} ne 'nofield' and $C{stone} ne 'nostone') {
        #----------------------------------------------------------------------
        # Release Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>', '');
        $C{mw}->bind('<Pause>', '');

        $C{mw}->bind('<Right>',    '');
        $C{mw}->bind('<Left>',     '');
        $C{mw}->bind('<Down>',     '');
        $C{mw}->bind('<Spacebar>', '');
        $C{mw}->bind('<Up>',       '');

        #----------------------------------------------------------------------
        # Delete all objects on the canvas:
        #----------------------------------------------------------------------
        $C{canvas}->delete('all');
        $C{objects} = [];

        #----------------------------------------------------------------------
        # Draw the grid:
        #----------------------------------------------------------------------
        draw_grid();

        #----------------------------------------------------------------------
        # Show the field and the stone:
        #----------------------------------------------------------------------
        show_field($C{field});
        $C{objects} = [ show_stone() ];

        #----------------------------------------------------------------------
        # Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>',     \&pause_game             );
        $C{mw}->bind('<Pause>',     \&pause_game             );

        $C{mw}->bind('<Right>',     [ \&move_stone, 'right' ]);
        $C{mw}->bind('<Left>',      [ \&move_stone, 'left'  ]);
        $C{mw}->bind('<Down>',      [ \&move_stone, 'down'  ]);
        $C{mw}->bind('<Key-space>', \&drop_stone             );
        $C{mw}->bind('<Up>',        [ \&move_stone, 'turn'  ]);

        #----------------------------------------------------------------------
        # Let the stone fall down in one second:
        #----------------------------------------------------------------------
        $C{after} = $C{mw}->after($C{speed}, \&fall_down);
    }

} # sub toggle_grid


sub pause_game {
    #--------------------------------------------------------------------------
    # Function     : p a u s e _ g a m e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Pauses the game.
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-22 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # Cancel drops
    #--------------------------------------------------------------------------
    $C{after}->cancel() if exists $C{after} and defined $C{after};

    #--------------------------------------------------------------------------
    # Release Bindings:
    #--------------------------------------------------------------------------
    $C{mw}->bind('<Right>',    '');
    $C{mw}->bind('<Left>',     '');
    $C{mw}->bind('<Down>',     '');
    $C{mw}->bind('<Spacebar>', '');
    $C{mw}->bind('<Up>',       '');

    #--------------------------------------------------------------------------
    # Delete all objects on the canvas:
    #--------------------------------------------------------------------------
    $C{canvas}->delete('all');
    $C{objects} = [];

    #--------------------------------------------------------------------------
    # Draw the grid:
    #--------------------------------------------------------------------------
    draw_grid();

    #--------------------------------------------------------------------------
    # Create the pause field:
    #--------------------------------------------------------------------------
    my @pausefield;
    for my $r (0..$C{rows}-1) {
        for my $c (0..$C{cols}-1) {
            $pausefield[$r][$c] = int rand $#Color;
        }
    }

    show_field(\@pausefield);

    #--------------------------------------------------------------------------
    # Bindings:
    #--------------------------------------------------------------------------
    $C{mw}->bind('<Key-p>', \&pause_game_doit );
    $C{mw}->bind('<Pause>', \&pause_game_doit );

    sub pause_game_doit {
        #----------------------------------------------------------------------
        # Release Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>', '');
        $C{mw}->bind('<Pause>', '');

        $C{mw}->bind('<Right>',    '');
        $C{mw}->bind('<Left>',     '');
        $C{mw}->bind('<Down>',     '');
        $C{mw}->bind('<Spacebar>', '');
        $C{mw}->bind('<Up>',       '');

        #----------------------------------------------------------------------
        # Delete all objects on the canvas:
        #----------------------------------------------------------------------
        $C{canvas}->delete('all');
        $C{objects} = [];

        #----------------------------------------------------------------------
        # Draw the grid:
        #----------------------------------------------------------------------
        draw_grid();

        #----------------------------------------------------------------------
        # Show the field and the stone:
        #----------------------------------------------------------------------
        show_field($C{field});
        $C{objects} = [ show_stone() ];

        #----------------------------------------------------------------------
        # Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>',     \&pause_game             );
        $C{mw}->bind('<Pause>',     \&pause_game             );

        $C{mw}->bind('<Right>',     [ \&move_stone, 'right' ]);
        $C{mw}->bind('<Left>',      [ \&move_stone, 'left'  ]);
        $C{mw}->bind('<Down>',      [ \&move_stone, 'down'  ]);
        $C{mw}->bind('<Key-space>', \&drop_stone             );
        $C{mw}->bind('<Up>',        [ \&move_stone, 'turn'  ]);

        #----------------------------------------------------------------------
        # Let the stone fall down in one second:
        #----------------------------------------------------------------------
        $C{after} = $C{mw}->after($C{speed}, \&fall_down);

    } # sub pause_game_doit

} # sub pause_game


sub fall_down {
    #--------------------------------------------------------------------------
    # Function     : f a l l _ d o w n
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Let the stone fall down
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Making the grid optional.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # Let the stone fall down one row:
    #--------------------------------------------------------------------------
    my $fall_ok = 1;
    for my $square (@{$C{stone}->coords()}) {
        my ($ri, $ci) = @$square;
        $fall_ok = 0, last if $ri >= $C{rows}-1 or $C{field}->[$ri+1][$ci];
    }

    #--------------------------------------------------------------------------
    # If that was ok, move the object on the canvas down and correct the
    # coordinates of the stone:
    #--------------------------------------------------------------------------
    if ($fall_ok) {
        $C{canvas}->move($_, 0, $C{rowpix} + $C{withgrid}) for @{$C{objects}};
        ++ $C{stone}->coords()->[$_]->[0] for 0 .. $#{ $C{stone}->coords() };
    }

    #--------------------------------------------------------------------------
    # Else write that stone into the field and create a new stone:
    #--------------------------------------------------------------------------
    else {
        #----------------------------------------------------------------------
        # Clear Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>', '');
        $C{mw}->bind('<Pause>', '');

        $C{mw}->bind('<Right>',    '');
        $C{mw}->bind('<Left>',     '');
        $C{mw}->bind('<Down>',     '');
        $C{mw}->bind('<Spacebar>', '');
        $C{mw}->bind('<Up>',       '');

        #----------------------------------------------------------------------
        # Write Stone into the field:
        #----------------------------------------------------------------------
        for my $square (@{$C{stone}->coords()}) {
            my ($ri, $ci) = @$square;
            $C{field}->[$ri][$ci] = $C{stone}->color();
        }
        $C{points} += 3 + $C{stone}->bonus(); # Add bonus for this stone
        $C{stone} = undef;

        #----------------------------------------------------------------------
        # Clear the full lines and increase the points:
        #----------------------------------------------------------------------
        clear_lines();

        #----------------------------------------------------------------------
        # Create a new stone
        #----------------------------------------------------------------------
        $C{stone} = new Pentris::Stone;
        $C{objects} = [ show_stone() ];

        #----------------------------------------------------------------------
        # Test if the coords of the new stone are free on the field
        #----------------------------------------------------------------------
        my $insert_ok = 1;
        for my $square (@{$C{stone}->coords()}) {
            my ($ri, $ci) = @$square;
            $insert_ok = 0, last if $C{field}->[$ri][$ci];
        }
        game_over(), return unless $insert_ok;

        #----------------------------------------------------------------------
        # Bindings:
        #----------------------------------------------------------------------
        $C{mw}->bind('<Key-p>',     \&pause_game             );
        $C{mw}->bind('<Pause>',     \&pause_game             );

        $C{mw}->bind('<Right>',     [ \&move_stone, 'right' ]);
        $C{mw}->bind('<Left>',      [ \&move_stone, 'left'  ]);
        $C{mw}->bind('<Down>',      [ \&move_stone, 'down'  ]);
        $C{mw}->bind('<Key-space>', \&drop_stone             );
        $C{mw}->bind('<Up>',        [ \&move_stone, 'turn'  ]);

    }

    #--------------------------------------------------------------------------
    # Let the stone fall down in one second:
    #--------------------------------------------------------------------------
    $C{after} = $C{mw}->after($C{speed}, \&fall_down);

} # sub fall_down


sub move_stone {
    #--------------------------------------------------------------------------
    # Function     : m o v e _ s t o n e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Move the stone (left, right, or turn)
    # Parameter    : 1) move ('right', 'left', 'drop', or 'turn')
    #                (all other are in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-22 - CD - Move one row done added.
    # 0.0.3 - 2004-08-24 - CD - Making the grid optional.
    # 0.0.4 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my (undef, # because of the bind
        $move) = @_;


    die "wrong call of move_stone"
        unless defined $move;

    #--------------------------------------------------------------------------
    # Get the stone coordinates:
    #--------------------------------------------------------------------------
    my @coord;
    for my $square (@{$C{stone}->coords()}) {
        my ($ri, $ci) = @$square;
        push @coord, [$ri, $ci];
    }

    #--------------------------------------------------------------------------
    # Calculate the new coordinates:
    #--------------------------------------------------------------------------
    my @newcoord;
    # center of gravity for turning of the coordinates is the first square:
    my ($center_row, $center_col) = @{ $coord[0] };
    for my $square (@coord) {
        my ($ri, $ci) = @$square;
        if ($move eq 'right') {
            ++$ci;
        }
        elsif ($move eq 'left') {
            --$ci;
        }
        elsif ($move eq 'turn') {
            ($ri, $ci) = ($center_row - $center_col + $ci,
                          $center_col + $center_row - $ri);
        }
        elsif ($move eq 'down') {
            ++$ri;
        }
        else {
            die "unknown move '$move'";
        }
        push @newcoord, [$ri, $ci];
    }

    #--------------------------------------------------------------------------
    # Check if it is ok to move:
    #--------------------------------------------------------------------------
    my $ok_to_move = 1;
    for my $square (@newcoord) {
        my ($ri, $ci) = @$square;
        $ok_to_move = 0, last if #$ri <  0     or # without this stones can be
                                                  # turned around earlier!
                                 $ri >= $C{rows} or
                                 $ci <  0     or
                                 $ci >= $C{cols} or
                                 $C{field}->[$ri][$ci];
    }

    #--------------------------------------------------------------------------
    # If that was ok, move the object on the canvas and correct the
    # coordinates of the stone:
    #--------------------------------------------------------------------------
    if ($ok_to_move) {
        for my $oi (0..$#{$C{objects}}) {
            my $delta_row = $newcoord[$oi]->[0] - $coord[$oi]->[0];
            my $delta_col = $newcoord[$oi]->[1] - $coord[$oi]->[1];
            $C{canvas}->move($C{objects}->[$oi],
                             ($C{colpix} + $C{withgrid}) * $delta_col,
                             ($C{rowpix} + $C{withgrid}) * $delta_row,
                            );
            $C{stone}->coords()->[$oi]->[0] += $delta_row;
            $C{stone}->coords()->[$oi]->[1] += $delta_col;
        }
    }

    return $ok_to_move;
} # sub move_stone


sub drop_stone {
    #--------------------------------------------------------------------------
    # Function     : d r o p _ s t o n e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : drop the stone
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Making the grid optional.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my $fall_ok = 1;

    while ($fall_ok) {
        #----------------------------------------------------------------------
        # Let the stone fall down one row:
        #----------------------------------------------------------------------
        for my $square (@{$C{stone}->coords()}) {
            my ($ri, $ci) = @$square;
            $fall_ok = 0, last if $ri >= $C{rows}-1 or $C{field}->[$ri+1][$ci];
        }

        #----------------------------------------------------------------------
        # If that was ok, move the object on the canvas down and correct the
        # coordinates of the stone:
        #----------------------------------------------------------------------
        if ($fall_ok) {
            $C{canvas}->move($_, 0, $C{rowpix} + $C{withgrid}) for @{$C{objects}};
            ++ $C{stone}->coords()->[$_]->[0] for 0 .. $#{ $C{stone}->coords() };
        }
    }
} # sub drop_stone


sub game_over {
    #--------------------------------------------------------------------------
    # Function     : g a m e _ o v e r
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Game is over, no more stones can be inserted.
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Catch the close-window-button
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # Game data:
    #--------------------------------------------------------------------------
    my $player = $ENV{USER} || 'unknown';
    my $points = $C{points};
    my $date;
    {
        my ($d, $m, $y) = (localtime)[3,4,5];

        ++$m;       # month are coded as 0..11
        $y += 1900; # years are ment as 1900 + year

        $date = sprintf "%04s-%02s-%02s", $y, $m, $d;
    }


    #--------------------------------------------------------------------------
    # Read the highscore file
    #--------------------------------------------------------------------------
    my @highscore;
    my $hfile = $C{gametype} eq 'Pentris'
                    ? $Config{highscore_p}
                    : $Config{highscore_t};

    if (open (HIGH, $hfile)) {
        chomp( @highscore = <HIGH> );
    }

    #--------------------------------------------------------------------------
    # Sort the highscores:
    #--------------------------------------------------------------------------
    @highscore = sort {my ($aplayer, $apoints, $adate) = split /\t/, $a;
                       my ($bplayer, $bpoints, $bdate) = split /\t/, $b;
                       $bpoints <=> $apoints
                      }
                      @highscore;

    #--------------------------------------------------------------------------
    # Look for the place in highscore list:
    #--------------------------------------------------------------------------
    my $place = 0;
    my $placetoday = 0;
    for (@highscore) {
        my ($hplayer, $hpoints, $hdate) = split /\t/;
        ++$place if $hpoints >= $points;
        ++$placetoday if $hpoints >= $points and $hdate eq $date;
    }


    #--------------------------------------------------------------------------
    # Show a dialog, that informs the user and asks for the name:
    #--------------------------------------------------------------------------
    {
        my $tl = $C{mw}->Toplevel();
        $tl->title('Spielende');
        $tl->Label(-text    => 'Herzlichen Glueckwunsch, Sie haben ' . $points .
                               ' Punkte erreicht.' . "\n" .
                               'Damit liegen Sie heute auf Platz ' .
                               ($placetoday+1) .
                               ' und  insgesamt auf Platz ' . ($place+1) . '.',
                   -justify => 'left',
                  )
            ->pack(-anchor  => 'w',
                  );
        $tl->Label(-text    => 'Bitte geben Sie Ihren Namen ein:',
                   -justify => 'left',
                  )
            ->pack(-anchor  => 'w');
        my $entry = $tl->Entry(-textvariable => \$player)
                        ->pack();
        my $ok = $tl->Button(-text        => 'OK',
                             -command     => sub {$tl->destroy();
                                                  $C{mw}->bind('<Escape>',
                                                               \&my_exit);
                                                  $C{mw}->focus();
                                                  game_over_doit($player,
                                                                 $points,
                                                                 $date,
                                                                 $hfile,
                                                                 $place,
                                                                 $placetoday,
                                                                 \@highscore,
                                                                );
                                                 },
                             -default     => 'active',
                             -padx        => 15,
                            )
                      ->pack(-expand      => '0',
                             -fill        => 'none',
                             -side        => 'right',
                            );

        #----------------------------------------------------------------------
        # Release bindings for <Escape> and bind them to delete this window:
        # (When the window is deleted, the binding of <Escape> is bound back
        # to my_exit(), see ok Button above.)
        #----------------------------------------------------------------------
        $C{mw}->bind('<Escape>', '');
        $tl->bind('<Escape>', sub{$ok->invoke()});
        $tl->bind('<Return>', sub{$ok->invoke()});

        #----------------------------------------------------------------------
        # Catch the close-window-button:
        #----------------------------------------------------------------------
        $tl->protocol("WM_DELETE_WINDOW" => sub { $ok->invoke() });

        #----------------------------------------------------------------------
        # Rest:
        #----------------------------------------------------------------------
        $tl->grab();
        $tl->focus();
        $entry->focus();
    }


    sub game_over_doit() {
        my ($player, $points, $date, $hfile, $place, $placetoday, $highscore)
           = @_;

        #----------------------------------------------------------------------
        # Add new highscore entry and write the highscore file:
        #----------------------------------------------------------------------
        push @$highscore, "$player\t$points\t$date";
        # Sort the highscores again:
        @$highscore = sort {my ($aplayer, $apoints, $adate) = split /\t/, $a;
                            my ($bplayer, $bpoints, $bdate) = split /\t/, $b;
                            $bpoints <=> $apoints
                           } @$highscore;
        my @highscore_today = grep {(undef, undef, $a) = split /\t/; $a eq $date}
                              @$highscore;
        if (open (HIGH, ">$hfile")) {
            print HIGH $_,"\n" for @$highscore;
        }
        else {
            die "Can't write the highscore file '$hfile'.";
        }


        #----------------------------------------------------------------------
        # Show highscore list:
        #----------------------------------------------------------------------
        my $text = "Spielende!\n".
                   "~~~~~~~~~~\n\n".
                   'Herzlichen Glueckwunsch '. $player. ', Sie haben ' . $points .
                   ' Punkte erreicht.' . "\n" .
                   'Damit liegen Sie heute auf Platz ' .
                   ($placetoday+1) .
                   ' und insgesamt auf Platz ' . ($place+1) . '.'."\n\n".
                   "Highscores fuer die Spielart $C{gametype}:\n";
        $text .= "\nHighscore heute\n" .
                 "~~~~~~~~~~~~~~~\n";
        $text .= sprintf "%3s: %s\n", $_+1, $highscore_today[$_]
            for 0..($#highscore_today<9 ? $#highscore_today : 9);
        $text .= "\nHighscore gesamt\n" .
                "~~~~~~~~~~~~~~~~\n";
        $text .= sprintf "%3s: %s\n", $_+1, $highscore->[$_]
            for 0..($#$highscore<9 ? $#$highscore : 9);

        show_window($C{mw},
                    'Spielende',
                    'Spielende',
                    '{Courier New} 10 {normal}',
                    $text,
                   );
    } # sub game_over_doit

} # sub game_over


sub show_highscore {
    #--------------------------------------------------------------------------
    # Function     : s h o w _ h i g h s c o r e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Shows the complete highscore lists.
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-22 - CD - Created
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my $date;
    {
        my ($d, $m, $y) = (localtime)[3,4,5];

        ++$m;       # month are coded as 0..11
        $y += 1900; # years are ment as 1900 + year

        $date = sprintf "%04s-%02s-%02s", $y, $m, $d;
    }

    #--------------------------------------------------------------------------
    # Read the highscore files
    #--------------------------------------------------------------------------
    my @highscore_p;
    my @highscore_t;
    my $hfile_p = $Config{highscore_p};
    my $hfile_t = $Config{highscore_t};
    if (open (HIGH, $hfile_p)) {
        chomp( @highscore_p = <HIGH> );
    }
    if (open (HIGH, $hfile_t)) {
        chomp( @highscore_t = <HIGH> );
    }
    my @highscore_today_p = grep {(undef, undef, $a) = split /\t/; $a eq $date}
                            @highscore_p;
    my @highscore_today_t = grep {(undef, undef, $a) = split /\t/; $a eq $date}
                            @highscore_t;

    #--------------------------------------------------------------------------
    # Sort the highscores:
    #--------------------------------------------------------------------------
    @highscore_p = sort {my ($aplayer, $apoints, $adate) = split /\t/, $a;
                         my ($bplayer, $bpoints, $bdate) = split /\t/, $b;
                         $bpoints <=> $apoints
                        } @highscore_p;
    @highscore_t = sort {my ($aplayer, $apoints, $adate) = split /\t/, $a;
                         my ($bplayer, $bpoints, $bdate) = split /\t/, $b;
                         $bpoints <=> $apoints
                        } @highscore_t;

    #--------------------------------------------------------------------------
    # Show highscore list:
    #--------------------------------------------------------------------------
    my $text = "Highscores fuer Pentris heute\n\n";
    $text .= sprintf "%3s: %s\n", $_+1, $highscore_today_p[$_]
        for 0..$#highscore_today_p;
    $text .= "\nHighscore fuer Pentris gesamt\n\n";
    $text .= sprintf "%3s: %s\n", $_+1, $highscore_p[$_]
        for 0..$#highscore_p;
    $text .= "\nHighscores fuer Tetris heute\n\n";
    $text .= sprintf "%3s: %s\n", $_+1, $highscore_today_t[$_]
        for 0..$#highscore_today_t;
    $text .= "\nHighscore fuer Tetris gesamt\n\n";
    $text .= sprintf "%3s: %s\n", $_+1, $highscore_t[$_]
        for 0..$#highscore_t;


    show_window($C{mw},
                'Highscores',
                'All highscores of Pentris and Tetris',
                '{Courier New} 10 {normal}',
                $text,
               );
} # sub show_highscore


sub show_field {
    #--------------------------------------------------------------------------
    # Function     : s h o w _ f i e l d
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Starts the game.
    # Parameter    : 1) reference of the field
    #                (the rest is in %C)
    # Return value : rectangle object
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my ($field) = @_;

    die "wrong call of show_field" unless defined $field;


    for my $ri (0..$C{rows}-1) {
        for my $ci (0..$C{cols}-1) {
            paint_square($ri, $ci, $Color[$field->[$ri][$ci]])
                if 0 != $field->[$ri][$ci];
        }
    }

} # sub show_field


sub show_stone {
    #--------------------------------------------------------------------------
    # Function     : s h o w _ s t o n e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Starts the game.
    # Parameter    : none (all in %C)
    # Return value : rectangle objects
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    # 0.0.2 - 2004-08-20 - CD - Returns now the Array, not a reference to an
    #                           anonymous array.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my @objects;
    for my $square (@{$C{stone}->coords()}) {
        my ($ri, $ci) = @$square;
        push @objects,
        paint_square($ri, $ci, $Color[$C{stone}->color()]);
    }

    return @objects;
} # sub show_stone


sub paint_square {
    #--------------------------------------------------------------------------
    # Function     : p a i n t _ s q u a r e
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Starts the game.
    # Parameter    : 1) row
    #                2) column
    #                3) color
    #                (the rest is in %C)
    # Return value : rectangle object
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-20 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Making the grid optional.
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my ($row, $col, $color) = @_;

    die "wrong call of paint_square"
        unless defined $row
           and defined $col
           and defined $color;

    my $rect = $C{canvas}->createRectangle(
        ($C{withgrid}+$C{colpix})*$col+1+$C{withgrid},
        ($C{withgrid}+$C{rowpix})*$row+1+$C{withgrid},
        ($C{withgrid}+$C{colpix})*($col+1),
        ($C{withgrid}+$C{rowpix})*($row+1),
        -fill    => $color,
        -outline => $color,
    );

    return $rect;

} # sub paint_square


sub my_exit {
    #--------------------------------------------------------------------------
    # Function     : m y _ e x i t
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Ask the user if he really want to exit and exit (or not).
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #                           askforexit added.
    #--------------------------------------------------------------------------

    if ($C{askforexit}) {
        #my $icon = $C{mw}->Photo(-file   => 'xyz.bmp',
        #                      -height => 32,
        #                      -width  => 32,
        #                     );
        my $dialog = $C{mw}->Dialog(-text           => "Wirklich beenden?",
                                    -bitmap         => 'question',
                                   #-image          => $icon,
                                    -title          => 'Pentris Beenden?',
                                    -default_button => 'OK',
                                    -buttons        => [ 'Abbruch', 'OK' ],
                                   #-font           => '{Courier New} 10 {normal}',
                                    -font           => '{Times New Roman} 12 {normal}',
                                   );
        my $answer = $dialog->Show(-global);

        exit if $answer eq 'OK';
    }
    else {
        exit;
    }

} # sub my_exit


sub show_window {
    #--------------------------------------------------------------------------
    # Function     : s h o w _ w i n d o w
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Shows the user some information in an extra window.
    # Parameter    : Necessary parameters:
    #                  1) Main window widget
    #                  2) title
    #                  3) label
    #                  4) font
    #                  5) text
    #                Optional:
    #                  6) width            (default: 80       )
    #                  7) height           (default: 35       )
    #                  8) text color       (default: 'blue'   )
    #                  9) background color (default: '#d7d7d7')
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-24 - CD - Catch the close-window-button.
    #--------------------------------------------------------------------------

    my $mw     = shift;
    my $title  = shift;
    my $label  = shift;
    my $font   = shift;
    my $text   = shift;
    my $width  = shift || 80;
    my $height = shift || 35;
    my $tcolor = shift || 'blue';
    my $bcolor = shift || '#d7d7d7';

    die "wrong call of show_window"
        unless defined $mw
           and defined $title
           and defined $label
           and defined $font
           and defined $text
           and defined $width
           and defined $height
           and defined $tcolor
           and defined $bcolor;

    #--------------------------------------------------------------------------
    # Create new toplevel window that shows the text:
    #--------------------------------------------------------------------------
    my $tl = $mw->Toplevel();
    $tl->title($title);
    $tl->Label(-text   => $label,
               #-anchor => 'w',
              )
        ->pack(-anchor => 'w');
    my $fr = $tl->Frame()
                 ->pack(-expand => '0',
                        -fill   => 'x',
                        -side   => 'bottom',
                       );
    my $txt = $tl->Scrolled("Text",
                            -scrollbars => 'osoe',
                            -background => $bcolor,
                            -foreground => $tcolor,
                            -state      => 'disabled',
                            -width      => $width,
                            -height     => $height,
                            -relief     => 'flat',
                            -font       => $font,
                            -wrap       => 'none',
                           )
                     ->pack(-side       => 'top',
                            -expand     => 1,
                            -fill       => 'both',
                           );
    my $ok = $fr->Button(-text        => 'OK',
                         -command     => sub {$tl->destroy();
                                              $mw->bind('<Escape>',
                                                        \&my_exit);
                                              $mw->focus();
                                             },
                         -default     => 'active',
                         -padx        => 15,
                        )
                  ->pack(-expand      => '0',
                         -fill        => 'none',
                         -side        => 'right',
                        );

    $txt->configure(-state => 'normal');
    $txt->delete('1.0', 'end');
    $txt->insert('end', $text);
    #$txt->insert('end', "\n");
    $txt->configure(-state => 'disabled');

    #--------------------------------------------------------------------------
    # Release bindings for <Escape> and bind them to delete this window:
    # (When the window is deleted, the binding of <Escape> is bound back to
    # my_exit(), see ok Button above.)
    #--------------------------------------------------------------------------
    $tl->bind('<Return>', sub{$ok->invoke()});
    $mw->bind('<Escape>', '');
    $tl->bind('<Escape>', sub{$ok->invoke()});
    #        *$text*->yviewScroll(*number, what*)
    #            This command adjust the view in the window up or down
    #            according to *number* and *what*. *Number* must be an
    #            integer. *What* must be either units or pages. If *what* is
    #            units, the view adjusts up or down by *number* lines on the
    #            display; if it is pages then the view adjusts by *number*
    #            screenfuls. If *number* is negative then earlier positions
    #            in the text become visible; if it is positive then later
    #            positions in the text become visible.
    $txt->bind('<Prior>', sub { $txt->yviewScroll(-1, 'units') } );
    $txt->bind('<Next>',  sub { $txt->yviewScroll( 1, 'units') } );
    #$txt->bind('<Home>',  sub { $txt->see('1.0') } );
    #$txt->bind('<End>',   sub { $txt->see('end') } );
    $txt->bind('<Home>',  sub { $txt->yviewMoveto(0) } );
    $txt->bind('<End>',   sub { $txt->yviewMoveto(1) } );
    # geht beides, unteres wohl etwas schneller
    $txt->bind('<Left>',  sub { $txt->yviewScroll(-1, 'units') } );
    $txt->bind('<Up>',    sub { $txt->yviewScroll(-1, 'units') } );
    $txt->bind('<Right>', sub { $txt->yviewScroll( 1, 'units') } );
    $txt->bind('<Down>',  sub { $txt->yviewScroll( 1, 'units') } );
    # irgendwas stimmt noch nicht ganz... da muessen eingebaute Funktionalitaeten
    # mit reinspielen...

    #$txt->bind('<F9>',    sub { $txt->yviewScroll(-1, 'units') } );
    #$txt->bind('<F10>',   sub { $txt->yviewScroll( 1, 'units') } );
    #$txt->bind('<F5>',    sub { $txt->yviewScroll(-1, 'pages') } );
    #$txt->bind('<F6>',    sub { $txt->yviewScroll( 1, 'pages') } );

    #--------------------------------------------------------------------------
    # Catch the close-window-button:
    #--------------------------------------------------------------------------
    $tl->protocol("WM_DELETE_WINDOW" => sub { $ok->invoke() });

    #--------------------------------------------------------------------------
    # Rest:
    #--------------------------------------------------------------------------
    $tl->grab();
    $tl->focus();
    $txt->focus();
} # sub show_window


sub help () {
    #--------------------------------------------------------------------------
    # Function     : h e l p
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Shows the help
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-22 - CD - text appended
    # 0.0.3 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------


    my $text = qq(
     Pentris   V$Config{version} vom $Config{version_date}
     ~~~~~~~

Pentris ist ein Tetrisclone mit erweitertem Steinset. Wer Tetris noch nicht
kennt: Es fallen Steine in einem Gitter von oben nach unten, man kann sie nach
rechts und nach links verschieben, man kann sie drehen und man kann sie fallen
lassen.

Ziel des Spieles ist es, so viele komplett gefuellte Reihen wie moeglich zu
erzeugen - jede gefuellte Reihe wird aus dem Gitter entfernt und alles ueber
der entfernten Reihe sinkt weiter nach unten.

Es gibt Punkte fuer das Ablegen der Steine (nach Kompliziertheit der Steinform)
und das Fuellen ganzer Reihen. Dabei ist es nicht egal, auf welcher Hoehe die
Reihen entfernt werden, je weiter oben die Reihe ist, desto mehr Punkte gibt
es. Wenn vier oder mehr Reihen auf einmal abgeraeumt werden, gibt es Bonus-
punkte.

Steuerung:

    Pfeil nach links : Stein nach links bewegen
    Pfeil nach rechts: Stein nach rechts bewegen
    Pfeil nach oben  : Stein drehen
    Pfeil nach unten : Stein eine Reihe nach unten bewegen
    Leertaste        : Stein fallen lassen

Ein neues Spiel wird mit F2 gestartet. Das aktuelle Spiel (falls gerade eins
laeuft) wird ohne Wertung abgebrochen. Man kann nun zwischen dem normalen
Pentris-Steinset und dem herkoemlichen Tetris-Steinset waehlen.

Man kann das Spiel mit den Tasten 'Pause' und 'p' pausieren und mit den
gleichen Tasten auch wieder weiterlaufen lassen. Waehrend der Pause wird ein
Zufallsmuster im Spielfeld angezeigt und es lassen sich auch keine Bewegungen
waehrend der Pause durchfuehren.

Mit F3 kann das Gitter an und ausgeschaltet werden.
);


    show_window($C{mw},
                'Hilfe zu Pentris',
                'Hilfe:',
                '{Courier New} 10 {normal}',
                $text,
               );

} # sub help


sub history () {
    #--------------------------------------------------------------------------
    # Function     : h i s t o r y
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Show the history
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    my $text = '
Version 0.0.4 (2004-08-25)
~~~~~~~~~~~~~

Version 0.0.3 (2004-08-24)
~~~~~~~~~~~~~

Version 0.0.2 (2004-08-23)
~~~~~~~~~~~~~

Version 0.0.1 (2004-08-20 bis 2004-08-22)
~~~~~~~~~~~~~

Programm erstellt, Steindefinitionen aus meinem alten Fortranprogramm
PENTRIS.FOR von 1993 uebernommen :-)

Falls das Spielfeld nicht auf den Monitor passt, bitte den Quellcode
folgendermaszen anpassen:

In der Belegung von Werten in %C :

          rowpix     => 31,
          colpix     => 31,

kleinere Werte eintragen. rowpix gibt an, wieviele Pixel eine Zeile hoch ist,
colpix gibt etsprechend die Breite der Spalten in Pixeln an.

%C findet man im Quellcode ganz am Anfang hinter "package main".
';

    show_window($C{mw},
                'Historie von Pentris',
                'Historie von Pentris:',
                '{Courier New} 10 {normal}',
                $text,
               );

} # sub history


sub over_pentris () {
    #--------------------------------------------------------------------------
    # Function     : o v e r _ p e n t r i s
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Tell the user something about this program.
    # Parameter    : none (all in %C)
    # Return value : none
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # History text:
    #--------------------------------------------------------------------------
    my $text = qq(
     Pentris
     ~~~~~~~

Version : V$Config{version} vom $Config{version_date}

Autor   : Christian Duehl
E-Mail  : crian AT perl-community DOT de
          christian AT duehl DOT de
Webseite: http://www.duehl.de/christian/perl/pentris.html

Pentris ist ein Tetris-Clone mit wahlweise mehr Steinen: Pentominos, also
alle Kombinationen mit fuenf Quadraten und zusaetzlich noch ein paar kleinere
Lueckenfueller, weil es sonst zu schwer wird.
);

    show_window($C{mw},
                'Informationen Ã¼ber Pentris',
                'Ãœber Pentris',
                '{Courier New} 10 {normal}',
                $text,
               );

} # sub over_pentris


sub clear_lines {
    #--------------------------------------------------------------------------
    # Function     : c l e a r _ l i n e s
    #--------------------------------------------------------------------------
    # Author       : Christian Duehl
    # Task         : Let the stone fall down
    # Parameter    : none (all in %C)
    # Return value : nothing
    #--------------------------------------------------------------------------
    # 0.0.1 - 2004-08-21 - CD - Created
    # 0.0.2 - 2004-08-22 - CD - Points adjusted
    # 0.0.3 - 2004-08-23 - CD - Bonus for totally cleared field added.
    # 0.0.4 - 2004-08-24 - CD - Bonus for totally cleared field added raised.
    # 0.0.5 - 2004-08-25 - CD - Holding all the Informations in %C instead of
    #                           moving them from function to function (this
    #                           was the only way to get toggle_grid running).
    #                           Bonus for four or more full lines increased.
    #--------------------------------------------------------------------------

    #--------------------------------------------------------------------------
    # Test for all rows, if they are full:
    #--------------------------------------------------------------------------
    my $cleared = 0;
    for my $row (0..$C{rows}-1) {
        my $rowfull = 1;
        #----------------------------------------------------------------------
        # Test for rows $row, if it is full:
        #----------------------------------------------------------------------
        for my $col (0..$C{cols}-1) {
            if (0 == $C{field}->[$row][$col]) {
                $rowfull = 0;
                last;
            }
        }
        #----------------------------------------------------------------------
        # If it is full: Clear that line and increase the points
        #----------------------------------------------------------------------
        if ($rowfull) {
            ++$cleared;
            $C{points} += 13*($C{rows}-$row)*($cleared+1);
            for my $row_t (reverse 1..$row) {
                for my $col (0..$C{cols}-1) {
                    $C{field}->[$row_t][$col] = $C{field}->[$row_t-1][$col];
                }
            }
            for my $col (0..$C{cols}-1) {
                $C{field}->[0][$col] = 0;
            }
        }
    }

    #--------------------------------------------------------------------------
    # Bonus for four or more full lines:
    #--------------------------------------------------------------------------
    #$C{points} += 47*$cleared if $cleared > 3;
    $C{points} += 512*($cleared-3) if $cleared > 3;

    #--------------------------------------------------------------------------
    # Bonus for totally cleared field:
    #--------------------------------------------------------------------------
    {
        my $all_clear = 1;
        for my $r (0..$C{rows}-1) {
            for my $c (0..$C{cols}-1) {
                $all_clear = 0, last if $C{field}->[$r][$c];
            }
        }
        #$C{points} = int (1.25 * $C{points}) if $all_clear;
        $C{points} *= 2 if $all_clear;
    }

    #--------------------------------------------------------------------------
    # Adjust game speed:
    #--------------------------------------------------------------------------
    $C{speed} -= $cleared * 10;

    #--------------------------------------------------------------------------
    # If lines are cleared delete all objects on the canvas and paint them
    # again:
    #--------------------------------------------------------------------------
    if ($cleared) {
        $C{canvas}->delete('all');
        $C{objects} = undef; # the stone was just added to the field
        draw_grid ();
        show_field($C{field});
    }

} # sub clear_lines

__END__

=head1 NAME

    pentris.pl - a Tetris clone with pentomino stone set.

=head SYNTAX

    pentris.pl [nogrid]

=head1 COPYRIGHT

    Copyright 2004 Christian Duehl. All rights reserved.

    This program is free software. You can redistribute it and/or
    modify it under the same terms as perl itself.

=head1 TODO

=over

=item optional show the next stone in extra widget

=item Pentris over the Net

=over 2

=item Variante A

    Beide haben ihr normales Pentrisfeld, bekommen
    aber die gleichen Steine. Zu klaeren ist, ob man das Feld und
    die Bewegungen des anderen sehen soll oder nicht, und ob
    gewartet werden soll, bis beide den Stein abgelegt haben,
    oder ob einfach die Steine, die einer schneller ablegt, in eine
    Schlange wandern und dem anderen halt spaeter vorgesetzt
    werden.
    Ich faende es lustig wenn man sieht, was der andere macht.
    Ansonsten koennte man ja auch einfach die Highscorelisten
    vergleichen und die STeinfolge irgendwie codieren...

=item Variante B

    A mit Zeitbeschraenkung, hier darf natuerlich nicht
    gewartet werden. Jeder bekommt die gleichen Steine, wer am
    meisten Punkte macht gewinnt.

=item Variante C

    kooperativ: Ein breites Spielfeld in dem beide an
    der gleichen Aufgabe basteln, der eine kann die Steine
    steuern, die auf der linken Seite fallen, der andere die auf der
    rechten Seite, Punkte gibt es zusammen.
    Die fallenden Steine sind sich im Weg und koennen natuerlich
    auch auf die Seite des Partners verschafft werden...

=item Variante D

    unkooperativ: Wie C, aber jeder hat sein eigenes
    Punktekonto, wer mit seinem Klotz Reihen abbaut, bekommt
    halt die Punkte...

    Ich weiss aber nicht, ob das ueberhaupt funktioniert, oder ob es
    Probleme macht, wenn beide quasi gleichzeitig landen.

    Auch macht es Probleme, wenn ein Stein landet und damit
    Reihen abgebaut werden und andere Steine weiter runter
    sacken und dabei den anderen fallenden Stein treffen.

    Aber ich hoffe, dass sich das regeln laesst.

=back

=back

=cut

