package de.duehl.pentris.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;

import static de.duehl.pentris.ui.PentrisGui.WINDOWS_10;

/**
 * Diese Klasse stellt einen Dialog dar, der aufgerufen wird, wenn der Benutzer
 * das Fenster schließt.
 *
 * @version 1.02     2019-07-09
 * @author Christian Dühl
 */

public class QuitDialog extends ModalDialogBase {

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private JFrame parentFrame;

    /** Ergebnis des Dialogs. */
    private boolean quit;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Bild des Programms.
     */
    public QuitDialog(Colorizer colorizer, JFrame parentFrame, Image programImage) {
        super(parentFrame.getLocation(), programImage, "", colorizer);
        this.parentFrame = parentFrame;
        quit = false;

        setKeyBindings();

        fillDialog();
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        setKeyBindingEnter(() ->  chooseToQuit());
        setKeyBindingEscape(() -> chooseToGoON());
    }

    /** Erstellt den Dialog. */
    @Override
    protected void populateDialog() {
        setUndecorated(true);

        add(createButtonPart(), BorderLayout.CENTER);

        setDialogLocation();
    }

    private void setDialogLocation() {
        if (parentFrame != null) {
            Point location = parentFrame.getLocation();
            int x = (int) location.getX();
            int y = (int) location.getY();
            setLocation(x + 265, y + 90);
            if (WINDOWS_10) {
                x += 6;
            }
        }
        else {
            setLocation(350, 300);
        }
    }

    /** Erzeugt den Bereich mit der Überschrift und den Buttons. */
    private Component createButtonPart() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());

        panel.add(createTitleLabel(), BorderLayout.NORTH);
        panel.add(createButtonPanel(), BorderLayout.CENTER);

        return panel;
    }

    private Component createTitleLabel() {
        JLabel label = new JLabel("Möchten sie das Programm wirklich beenden?");
        label.setBorder(new EmptyBorder(5, 10, 5, 10));
        setColors(label);
        return label;
    }

    private Component createButtonPanel() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 10));

        panel.add(createYesButton());
        panel.add(createNoButton());

        return panel;
    }

    private Component createYesButton() {
        JButton button = createButton("Ja");
        button.addActionListener(e -> chooseToQuit());
        return button;
    }

    private Component createNoButton() {
        JButton button = createButton("Nein");
        button.addActionListener(e -> chooseToGoON());
        return button;
    }

    private JButton createButton(String title) {
        JButton button = new JButton(title);
        button.setFocusable(false);
        setColors(button);
        return button;
    }

    /** Wählt als Ergebnis des Dialogs, weiter zu machen. */
    private void chooseToGoON() {
        quit = false;
        closeDialog();
    }

    /** Wählt als Ergebnis des Dialogs, das Spiel zu beenden. */
    private void chooseToQuit() {
        quit = true;
        closeDialog();
    }

    /**
     * Ermittelt, ob wirklich beendet werden soll.
     *
     * @return true: Wirklich beenden.
     */
    public boolean doQuit() {
        return quit;
    }

}
