package de.duehl.basics.collections;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.junit.Test;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.system.OutputStreamSaveAndRelease;
import de.duehl.basics.text.Text;

public class CollectionsHelperTest {

    @Test
    public void getSortedIndicesForEmptyKeySet() {
        Set<Integer> keySet = new HashSet<>();
        List<Integer> list = CollectionsHelper.getSortedIndices(keySet);
        assertTrue(list.isEmpty());
    }

    @Test
    public void getSortedIndicesForOneNumber() {
        Set<Integer> keySet = new HashSet<>();
        Integer zero = Integer.valueOf(0);
        keySet.add(zero);
        List<Integer> list = CollectionsHelper.getSortedIndices(keySet);
        assertEquals(1, list.size());
        assertEquals(zero, list.get(0));
    }

    @Test
    public void getSortedIndicesForTwoNumberInsertedCorrectlySorted() {
        Set<Integer> keySet = new HashSet<>();
        Integer zero = Integer.valueOf(0);
        Integer one = Integer.valueOf(1);
        keySet.add(zero);
        keySet.add(one);
        List<Integer> list = CollectionsHelper.getSortedIndices(keySet);
        assertEquals(2, list.size());
        assertEquals(zero, list.get(0));
        assertEquals(one, list.get(1));
    }

    @Test
    public void getSortedIndicesForTwoNumberInsertedIncorrectlySorted() {
        Set<Integer> keySet = new HashSet<>();
        Integer zero = Integer.valueOf(0);
        Integer one = Integer.valueOf(1);
        keySet.add(one);
        keySet.add(zero);
        List<Integer> list = CollectionsHelper.getSortedIndices(keySet);
        assertEquals(2, list.size());
        assertEquals(zero, list.get(0));
        assertEquals(one, list.get(1));
    }

    @Test
    public void getSortedIndicesForFiveNumber() {
        Integer zero = Integer.valueOf(0);
        Integer one = Integer.valueOf(1);
        Integer two = Integer.valueOf(2);
        Integer three = Integer.valueOf(3);
        Integer four = Integer.valueOf(4);

        Set<Integer> keySet = new HashSet<>();
        keySet.add(one);
        keySet.add(four);
        keySet.add(zero);
        keySet.add(two);
        keySet.add(three);

        List<Integer> list = CollectionsHelper.getSortedIndices(keySet);

        assertEquals(5, list.size());
        assertEquals(zero, list.get(0));
        assertEquals(one, list.get(1));
        assertEquals(two, list.get(2));
        assertEquals(three, list.get(3));
        assertEquals(four, list.get(4));
    }

    @Test
    public void getSortedMapIndicesForFiveNumber() {
        Integer zero = Integer.valueOf(0);
        Integer one = Integer.valueOf(1);
        Integer two = Integer.valueOf(2);
        Integer three = Integer.valueOf(3);
        Integer four = Integer.valueOf(4);

        Map<Integer, String> map = new HashMap<>();
        map.put(three, "three");
        map.put(four, "four");
        map.put(one, "one");
        map.put(zero, "zero");
        map.put(two, "two");

        List<Integer> list = CollectionsHelper.getSortedMapIndices(map);

        assertEquals(5, list.size());
        assertEquals(zero, list.get(0));
        assertEquals(one, list.get(1));
        assertEquals(two, list.get(2));
        assertEquals(three, list.get(3));
        assertEquals(four, list.get(4));
    }

    @Test
    public void getSortedStringIndicesForEmptyKeySet() {
        Set<String> keySet = new HashSet<>();
        List<String> list = CollectionsHelper.getSortedStringIndices(keySet);
        assertTrue(list.isEmpty());
    }

    @Test
    public void getSortedStringIndicesForOneNumber() {
        Set<String> keySet = new HashSet<>();
        String zero = "zero";
        keySet.add(zero);
        List<String> list = CollectionsHelper.getSortedStringIndices(keySet);
        assertEquals(1, list.size());
        assertEquals(zero, list.get(0));
    }

    @Test
    public void getSortedStringIndicesForTwoNumberInsertedCorrectlySorted() {
        Set<String> keySet = new HashSet<>();
        String zero = "zero";
        String one = "one";
        keySet.add(one);
        keySet.add(zero);
        List<String> list = CollectionsHelper.getSortedStringIndices(keySet);
        assertEquals(2, list.size());
        assertEquals(zero, list.get(1));
        assertEquals(one, list.get(0));
    }

    @Test
    public void getSortedStringIndicesForTwoNumberInsertedIncorrectlySorted() {
        Set<String> keySet = new HashSet<>();
        String zero = "zero";
        String one = "one";
        keySet.add(zero);
        keySet.add(one);
        List<String> list = CollectionsHelper.getSortedStringIndices(keySet);
        assertEquals(2, list.size());
        assertEquals(zero, list.get(1));
        assertEquals(one, list.get(0));
    }

    @Test
    public void getSortedStringIndicesForFiveStrings() {
        String zero = "zero";
        String one = "one";
        String two = "two";
        String three = "three";
        String four = "four";

        Set<String> keySet = new HashSet<>();
        keySet.add(one);
        keySet.add(four);
        keySet.add(zero);
        keySet.add(two);
        keySet.add(three);

        List<String> list = CollectionsHelper.getSortedStringIndices(keySet);

        assertEquals(5, list.size());
        assertEquals(zero, list.get(4));
        assertEquals(one, list.get(1));
        assertEquals(two, list.get(3));
        assertEquals(three, list.get(2));
        assertEquals(four, list.get(0));
    }

    @Test
    public void getSortedMapStringIndicesForFiveNumber() {
        String zero = "zero";
        String one = "one";
        String two = "two";
        String three = "three";
        String four = "four";

        Map<String, String> map = new HashMap<>();
        map.put(three, "drei");
        map.put(four, "vier");
        map.put(one, "eins");
        map.put(zero, "null");
        map.put(two, "zwei");

        List<String> list = CollectionsHelper.getSortedMapStringIndices(map);

        assertEquals(5, list.size());
        assertEquals(zero, list.get(4));
        assertEquals(one, list.get(1));
        assertEquals(two, list.get(3));
        assertEquals(three, list.get(2));
        assertEquals(four, list.get(0));
    }

    @Test
    public void reverseList() {
        List<String> list = new ArrayList<>();
        list.add("zero");
        list.add("one");
        list.add("two");
        list.add("three");
        list.add("four");

        assertEquals(5, list.size());
        assertEquals("zero", list.get(0));
        assertEquals("one", list.get(1));
        assertEquals("two", list.get(2));
        assertEquals("three", list.get(3));
        assertEquals("four", list.get(4));

        List<String> reverse = CollectionsHelper.reverseList(list);

        assertEquals(5, list.size());
        assertEquals("zero", list.get(0));
        assertEquals("one", list.get(1));
        assertEquals("two", list.get(2));
        assertEquals("three", list.get(3));
        assertEquals("four", list.get(4));

        assertEquals(5, reverse.size());
        assertEquals("four", reverse.get(0));
        assertEquals("three", reverse.get(1));
        assertEquals("two", reverse.get(2));
        assertEquals("one", reverse.get(3));
        assertEquals("zero", reverse.get(4));
    }

    @Test
    public void listListOfStrings() {
        List<String> list = createStringlist();
        String actual = CollectionsHelper.listListOfStrings(list);
        String expected = createExpectedOutput();
        assertEquals(expected, actual);
    }

    @Test
    public void printListOfStrings() {
        List<String> list = createStringlist();
        OutputStreamSaveAndRelease ossar = new OutputStreamSaveAndRelease();
        ossar.saveSystemOut();
        CollectionsHelper.printListOfStrings(list);
        String actual = ossar.getOutContents();
        ossar.releaseSystemOut();
        String expected = createExpectedOutput();
        assertEquals(expected, actual);
    }

    private List<String> createStringlist() {
        List<String> list = new ArrayList<>();

        list.add("zero");
        list.add("one");
        list.add("two");
        list.add("three");
        list.add("four");

        return list;
    }

    private String createExpectedOutput() {
        return ""
                + "     0. [zero]" + Text.LINE_BREAK
                + "     1. [one]" + Text.LINE_BREAK
                + "     2. [two]" + Text.LINE_BREAK
                + "     3. [three]" + Text.LINE_BREAK
                + "     4. [four]" + Text.LINE_BREAK
                ;
    }

    @Test
    public void buildListFrom() {
        List<String> texts = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        assertEquals(3, texts.size());
        assertEquals("foo", texts.get(0));
        assertEquals("bar", texts.get(1));
        assertEquals("baz", texts.get(2));
    }

    @Test
    public void buildListFromEmptyList() {
        List<String> texts = CollectionsHelper.buildListFrom();
        assertEquals(0, texts.size());
    }

    @Test
    public void buildListFromOneElement() {
        List<String> texts = CollectionsHelper.buildListFrom("foo");
        assertEquals(1, texts.size());
        assertEquals("foo", texts.get(0));
    }

    @Test
    public void buildIntListFromWithoutInList() {
        List<Integer> values = CollectionsHelper.buildListFrom(1, 2, 3);
        assertEquals(3, values.size());
        assertEquals(Integer.valueOf(1), values.get(0));
        assertEquals(Integer.valueOf(2), values.get(1));
        assertEquals(Integer.valueOf(3), values.get(2));
    }

    @SuppressWarnings("deprecation")
    @Test
    public void buildIntListFrom() {
        List<Integer> values = CollectionsHelper.buildIntListFrom(1, 2, 3);
        assertEquals(3, values.size());
        assertEquals(Integer.valueOf(1), values.get(0));
        assertEquals(Integer.valueOf(2), values.get(1));
        assertEquals(Integer.valueOf(3), values.get(2));
    }

    @SuppressWarnings("deprecation")
    @Test
    public void buildIntListFromEmptyList() {
        List<Integer> values = CollectionsHelper.buildIntListFrom();
        assertEquals(0, values.size());
    }

    @SuppressWarnings("deprecation")
    @Test
    public void buildIntListFromOneElement() {
        List<Integer> values = CollectionsHelper.buildIntListFrom(1);
        assertEquals(1, values.size());
        assertEquals(Integer.valueOf(1), values.get(0));
    }

    @Test
    public void buildLoweredListFrom() {
        List<String> texts = CollectionsHelper.buildLoweredListFrom("FOO", "bar", "Baz");
        assertEquals(3, texts.size());
        assertEquals("foo", texts.get(0));
        assertEquals("bar", texts.get(1));
        assertEquals("baz", texts.get(2));
    }

    @Test
    public void buildLoweredListFromEmptyList() {
        List<String> texts = CollectionsHelper.buildLoweredListFrom();
        assertEquals(0, texts.size());
    }

    @Test
    public void buildLoweredListFromOneElement() {
        List<String> texts = CollectionsHelper.buildLoweredListFrom("fOo");
        assertEquals(1, texts.size());
        assertEquals("foo", texts.get(0));
    }

    @Test
    public void buildSortByLengthDescandingListFrom() {
        List<String> texts = CollectionsHelper.buildSortByLengthDescandingListFrom("bb", "a", "ccc");
        assertEquals(3, texts.size());
        assertEquals("ccc", texts.get(0));
        assertEquals("bb", texts.get(1));
        assertEquals("a", texts.get(2));

    }

    @Test
    public void buildSortByLengthListFrom() {
        List<String> texts = CollectionsHelper.buildSortByLengthListFrom("bb", "a", "ccc");
        assertEquals(3, texts.size());
        assertEquals("a", texts.get(0));
        assertEquals("bb", texts.get(1));
        assertEquals("ccc", texts.get(2));
    }

    @Test
    public void buildSortedListFrom() {
        List<String> texts = CollectionsHelper.buildSortedListFrom("foo", "bar", "baz");
        assertEquals(3, texts.size());
        assertEquals("bar", texts.get(0));
        assertEquals("baz", texts.get(1));
        assertEquals("foo", texts.get(2));
    }

    @Test
    public void buildDescandingSortedListFrom() {
        List<String> texts = CollectionsHelper.buildDescandingSortedListFrom("foo", "bar", "baz");
        assertEquals(3, texts.size());
        assertEquals("foo", texts.get(0));
        assertEquals("baz", texts.get(1));
        assertEquals("bar", texts.get(2));
    }

    @Test
    public void removeEmptyStringsFromListWithoutEmptyElements() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "c");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(3, list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
        assertEquals("c", list.get(2));
    }

    @Test
    public void removeEmptyStringsFromListFromEmptyList() {
        List<String> list = new ArrayList<>();
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsFromListWithEmptyElementAtFront() {
        List<String> list = CollectionsHelper.buildListFrom("", "b", "c");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("b", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithEmptyElementAtMiddle() {
        List<String> list = CollectionsHelper.buildListFrom("a", "", "c");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithEmptyElementAtEnd() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithMultipleEmptyElementAtFront() {
        List<String> list = CollectionsHelper.buildListFrom("", "", "", "", "", "", "b", "c");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("b", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithMultipleEmptyElementAtMiddle() {
        List<String> list = CollectionsHelper.buildListFrom("a", "", "", "", "c");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithMultipleEmptyElementAtEnd() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "", "", "");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
    }

    @Test
    public void removeEmptyStringsFromListWithOnlyEmptyElements() {
        List<String> list = CollectionsHelper.buildListFrom("", "", "", "", "");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsFromListWithOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("");
        CollectionsHelper.removeEmptyStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListFromEmptyList() {
        List<String> list = new ArrayList<>();
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithEmptyElementAtFront() {
        List<String> list = CollectionsHelper.buildListFrom("   ", "b", "c");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("b", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithEmptyElementAtMiddle() {
        List<String> list = CollectionsHelper.buildListFrom("a", "    ", "c");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithEmptyElementAtEnd() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "  \t   ");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithMultipleEmptyElementAtFront() {
        List<String> list = CollectionsHelper.buildListFrom(" ", "  ", "", "", "", "  ", "b", "c");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("b", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithMultipleEmptyElementAtMiddle() {
        List<String> list = CollectionsHelper.buildListFrom("a", "  ", "  ", "  ", "c");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("c", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithMultipleEmptyElementAtEnd() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", " ", "  ", "\t     ");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(2, list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithOnlyEmptyElements() {
        List<String> list = CollectionsHelper.buildListFrom("    ", "", "\t", "", "       ");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyAndOnlyWhitespaceStringsFromListWithOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("      ");
        CollectionsHelper.removeEmptyAndOnlyWhitespaceStringsFromList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeMultipleValues() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "a", "c", "a");
        List<String> singles = CollectionsHelper.removeMultipleValues(list);
        assertEquals(3,  singles.size());
        assertEquals("a", singles.get(0));
        assertEquals("b", singles.get(1));
        assertEquals("c", singles.get(2));
    }

    @Test
    public void removeMultipleValuesEmptyList() {
        List<String> list = CollectionsHelper.buildListFrom();
        List<String> singles = CollectionsHelper.removeMultipleValues(list);
        assertEquals(0,  singles.size());
    }

    @Test
    public void removeMultipleValuesOneElement() {
        List<String> list = CollectionsHelper.buildListFrom("a");
        List<String> singles = CollectionsHelper.removeMultipleValues(list);
        assertEquals(1,  singles.size());
        assertEquals("a", singles.get(0));
    }

    @Test
    public void removeMultipleValuesOneMultipleElement() {
        List<String> list = CollectionsHelper.buildListFrom("a", "a", "a", "a", "a", "a", "a");
        List<String> singles = CollectionsHelper.removeMultipleValues(list);
        assertEquals(1,  singles.size());
        assertEquals("a", singles.get(0));
    }

    @Test
    public void removeLastListEntry() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "c", "d", "e");
        CollectionsHelper.removeLastListEntry(list);
        assertEquals(4,  list.size());
        assertEquals("a", list.get(0));
        assertEquals("b", list.get(1));
        assertEquals("c", list.get(2));
        assertEquals("d", list.get(3));
    }

    @Test
    public void removeLastListEntryEmptyList() {
        List<String> list = CollectionsHelper.buildListFrom();
        CollectionsHelper.removeLastListEntry(list);
        assertEquals(0,  list.size());
    }

    @Test
    public void removeLastListEntryOneElement() {
        List<String> list = CollectionsHelper.buildListFrom("a");
        CollectionsHelper.removeLastListEntry(list);
        assertEquals(0,  list.size());
    }

    @Test
    public void stringArrayToList() {
        String[] array = { "a", "b", "c", "d", "e", "f", "g" };
        List<String> list = CollectionsHelper.stringArrayToList(array);
        checkListAndArrayAreEqual(list, array);
    }

    @Test
    public void stringListToArray01() {
        List<String> list = CollectionsHelper.buildListFrom("a", "b", "c", "d", "e", "f", "g");
        String[] array = CollectionsHelper.stringListToArray(list);
        checkListAndArrayAreEqual(list, array);
    }

    private void checkListAndArrayAreEqual(List<String> list, String[] array) {
        assertEquals(list.size(), array.length);
        for (int index = 0; index< list.size(); ++index) {
            assertEquals(list.get(index), array[index]);
        }
    }

    @Test
    public void stringListToArrayWithEmptyList() {
        List<String> list = new ArrayList<>();
        String[] array = CollectionsHelper.stringListToArray(list);
        checkListAndArrayAreEqual(list, array);
    }

    @Test
    public void stringListToArrayWithEmptyList2() {
        List<String> list = CollectionsHelper.buildListFrom();
        String[] array = CollectionsHelper.stringListToArray(list);
        checkListAndArrayAreEqual(list, array);
    }

    @Test
    public void stringListToArrayWithEmptyOneElement() {
        List<String> list = CollectionsHelper.buildListFrom("foo");
        String[] array = CollectionsHelper.stringListToArray(list);
        checkListAndArrayAreEqual(list, array);
    }

    @Test
    public void arrayToStringEmptyArray() {
        String[] array = {};
        String actual = CollectionsHelper.stringArrayToString(array);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void arrayToStringOneElement() {
        String[] array = { "foo" };
        String actual = CollectionsHelper.stringArrayToString(array);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void arrayToStringTwoElements() {
        String[] array = { "foo", "bar" };
        String actual = CollectionsHelper.stringArrayToString(array);
        String expected = "foo, bar";
        assertEquals(expected, actual);
    }

    @Test
    public void arrayToStringThreeElements() {
        String[] array = { "foo", "bar", "baz" };
        String actual = CollectionsHelper.stringArrayToString(array);
        String expected = "foo, bar, baz";
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfFirstNotEmptyElementWhenFirstElementIsNotEmpty() {
        List<String> list = CollectionsHelper.buildListFrom("foo");
        int actual = CollectionsHelper.determineIndexOfFirstNotEmptyElement(list);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfFirstNotEmptyElementWhenListIsEmpty() {
        List<String> list = CollectionsHelper.buildListFrom();
        int actual = CollectionsHelper.determineIndexOfFirstNotEmptyElement(list);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfFirstNotEmptyElementWhenListHasSizeOneAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("");
        int actual = CollectionsHelper.determineIndexOfFirstNotEmptyElement(list);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfFirstNotEmptyElementWhenListHasSizeTwoAndTwoEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "");
        int actual = CollectionsHelper.determineIndexOfFirstNotEmptyElement(list);
        int expected = 2;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfFirstNotEmptyElementWhenListHasSizeTwoAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "foo");
        int actual = CollectionsHelper.determineIndexOfFirstNotEmptyElement(list);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyStringsAtFrontOfListWhenFirstElementIsNotEmpty() {
        List<String> list = CollectionsHelper.buildListFrom("foo");
        CollectionsHelper.removeEmptyStringsAtFrontOfList(list);
        assertEquals(1, list.size());
        assertEquals("foo", list.get(0));
    }

    @Test
    public void removeEmptyStringsAtFrontOfListWhenListIsEmpty() {
        List<String> list = CollectionsHelper.buildListFrom();
        CollectionsHelper.removeEmptyStringsAtFrontOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtFrontOfListWhenListHasSizeOneAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("");
        CollectionsHelper.removeEmptyStringsAtFrontOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtFrontOfListWhenListHasSizeTwoAndTwoEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "");
        CollectionsHelper.removeEmptyStringsAtFrontOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtFrontOfListWhenListHasSizeTwoAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "foo");
        CollectionsHelper.removeEmptyStringsAtFrontOfList(list);
        assertEquals(1, list.size());
        assertEquals("foo", list.get(0));
    }

    @Test
    public void determineIndexOfLastNotEmptyElementWhenLastElementIsNotEmpty() {
        List<String> list = CollectionsHelper.buildListFrom("foo");
        int actual = CollectionsHelper.determineIndexOfLastNotEmptyElement(list);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfLastNotEmptyElementWhenListIsEmpty() {
        List<String> list = CollectionsHelper.buildListFrom();
        int actual = CollectionsHelper.determineIndexOfLastNotEmptyElement(list);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfLastNotEmptyElementWhenListHasSizeOneAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("");
        int actual = CollectionsHelper.determineIndexOfLastNotEmptyElement(list);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfLastNotEmptyElementWhenListHasSizeTwoAndTwoEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "");
        int actual = CollectionsHelper.determineIndexOfLastNotEmptyElement(list);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void determineIndexOfLastNotEmptyElementWhenListHasSizeTwoAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "");
        int actual = CollectionsHelper.determineIndexOfLastNotEmptyElement(list);
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void removeEmptyStringsAtEndOfListWhenLastElementIsNotEmpty() {
        List<String> list = CollectionsHelper.buildListFrom("foo");
        CollectionsHelper.removeEmptyStringsAtEndOfList(list);
        assertEquals(1, list.size());
        assertEquals("foo", list.get(0));
    }

    @Test
    public void removeEmptyStringsAtEndOfListWhenListIsEmpty() {
        List<String> list = CollectionsHelper.buildListFrom();
        CollectionsHelper.removeEmptyStringsAtEndOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtEndOfListWhenListHasSizeOneAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("");
        CollectionsHelper.removeEmptyStringsAtEndOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtEndOfListWhenListHasSizeTwoAndTwoEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("", "");
        CollectionsHelper.removeEmptyStringsAtEndOfList(list);
        assertEquals(0, list.size());
    }

    @Test
    public void removeEmptyStringsAtEndOfListWhenListHasSizeTwoAndOneEmptyElement() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "");
        CollectionsHelper.removeEmptyStringsAtEndOfList(list);
        assertEquals(1, list.size());
        assertEquals("foo", list.get(0));
    }

    @Test
    public void sortStringListByLengthDescandingAlphabeticalOnEqualLengthElements() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        CollectionsHelper.sortStringListByLengthDescanding(list);
        assertEquals(3, list.size());
        assertEquals("bar", list.get(0));
        assertEquals("baz", list.get(1));
        assertEquals("foo", list.get(2));
    }

    @Test
    public void sortStringListByLengthDescandingOnDifferentLength() {
        List<String> list = CollectionsHelper.buildListFrom("aa", "b", "ccc");
        CollectionsHelper.sortStringListByLengthDescanding(list);
        assertEquals(3, list.size());
        assertEquals("ccc", list.get(0));
        assertEquals("aa", list.get(1));
        assertEquals("b", list.get(2));
    }

    @Test
    public void sortStringListByLengthDescandingOnMixedData() {
        List<String> list = CollectionsHelper.buildListFrom("aa", "b", "foo", "bar", "baz", "ccc",
                "dddd");
        CollectionsHelper.sortStringListByLengthDescanding(list);
        assertEquals(7, list.size());
        assertEquals("dddd", list.get(0));
        assertEquals("bar", list.get(1));
        assertEquals("baz", list.get(2));
        assertEquals("ccc", list.get(3));
        assertEquals("foo", list.get(4));
        assertEquals("aa", list.get(5));
        assertEquals("b", list.get(6));
    }

    @Test
    public void isDisjunctString() {
        List<String> list = CollectionsHelper.buildListFrom("aa", "b", "foo", "bar", "baz", "ccc",
                "dddd");
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void isDisjunctInt() {
        List<Integer> list = CollectionsHelper.buildListFrom(1, 2, -5, 13);
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void isDisjunctObject() {
        List<Object> list = new ArrayList<>();
        list.add("1");
        list.add(1);
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = true;
        assertEquals(expected, actual);
    }

    @Test
    public void isDisjunctStringFailes() {
        List<String> list = CollectionsHelper.buildListFrom("aa", "b", "foo", "bar", "baz", "ccc",
                "dddd", "foo");
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = false;
        assertEquals(expected, actual);
    }

    @Test
    public void isDisjunctIntFailes() {
        List<Integer> list = CollectionsHelper.buildListFrom(1, 2, -5, 13, -5);
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = false;
        assertEquals(expected, actual);
    }

    @Test
    public void isDisjunctObjectFailes() {
        List<Object> list = new ArrayList<>();
        list.add("1");
        list.add(1);
        list.add("1");
        boolean actual = CollectionsHelper.isDisjunct(list);
        boolean expected = false;
        assertEquals(expected, actual);
    }

    @Test
    public void createDisjunctLowerCaseList() {
        List<String> list = CollectionsHelper.buildListFrom("B", "a", "C", "A", "b", "D");
        List<String> disjunctLowerCaseList = CollectionsHelper.createDisjunctLowerCaseList(list);
        assertEquals(4, disjunctLowerCaseList.size());
        assertEquals("b", disjunctLowerCaseList.get(0));
        assertEquals("a", disjunctLowerCaseList.get(1));
        assertEquals("c", disjunctLowerCaseList.get(2));
        assertEquals("d", disjunctLowerCaseList.get(3));
    }

    @Test
    public void createDisjunctListFromDisjunctList() {
        List<String> list = CollectionsHelper.buildListFrom("c", "a", "b");
        List<String> disjunctList = CollectionsHelper.createDisjunctList(list);
        assertEquals(3, disjunctList.size());
        assertEquals("c", disjunctList.get(0));
        assertEquals("a", disjunctList.get(1));
        assertEquals("b", disjunctList.get(2));
    }

    @Test
    public void createDisjunctListFormNotDisjunctList() {
        List<String> list = CollectionsHelper.buildListFrom("c", "a", "c", "a", "b", "b", "a");
        List<String> disjunctList = CollectionsHelper.createDisjunctList(list);
        assertEquals(3, disjunctList.size());
        assertEquals("c", disjunctList.get(0));
        assertEquals("a", disjunctList.get(1));
        assertEquals("b", disjunctList.get(2));
    }

    @Test
    public void createDisjunctListFormNotDisjunctIntList() {
        List<Integer> list = CollectionsHelper.buildListFrom(3, 1, 3, 1, 2, 2, 1);
        List<Integer> disjunctList = CollectionsHelper.createDisjunctList(list);
        assertEquals(3, disjunctList.size());
        assertEquals(Integer.valueOf(3), disjunctList.get(0));
        assertEquals(Integer.valueOf(1), disjunctList.get(1));
        assertEquals(Integer.valueOf(2), disjunctList.get(2));
    }

    @Test
    public void createDisjunctListFormNotDisjunctListsOfLists() {
        List<String> list1 = CollectionsHelper.buildListFrom("a", "b", "c");
        List<String> list2 = CollectionsHelper.buildListFrom("c", "a", "c");
        List<String> list3 = CollectionsHelper.buildListFrom("a", "b", "c");
        List<String> list4 = CollectionsHelper.buildListFrom("c", "b", "a");
        List<List<String>> list = new ArrayList<>();
        list.add(list1);
        list.add(list2);
        list.add(list3);
        list.add(list4);
        List<List<String>> disjunctList = CollectionsHelper.createDisjunctList(list);
        assertEquals(3, disjunctList.size());
        assertEquals(list1, disjunctList.get(0));
        assertEquals(list2, disjunctList.get(1));
        assertEquals(list4, disjunctList.get(2));
    }

    @Test
    public void toLowerCase() {
        List<String> list = CollectionsHelper.buildListFrom("Aaa", "A", "BbbBbb", "B", "cCcc", "d",
                "D", "EeeeE");
        List<String> loweredList = CollectionsHelper.toLowerCase(list);
        assertEquals(8, loweredList.size());
        assertEquals("aaa", loweredList.get(0));
        assertEquals("a", loweredList.get(1));
        assertEquals("bbbbbb", loweredList.get(2));
        assertEquals("b", loweredList.get(3));
        assertEquals("cccc", loweredList.get(4));
        assertEquals("d", loweredList.get(5));
        assertEquals("d", loweredList.get(6));
        assertEquals("eeeee", loweredList.get(7));
    }

    @Test
    public void sortAndToLowerCase() {
        List<String> list = CollectionsHelper.buildSortByLengthDescandingListFrom("Aaa", "A",
                "BbbBbb", "B", "cCcc", "d", "D", "EeeeE");
        List<String> loweredList = CollectionsHelper.toLowerCase(list);
        assertEquals(8, loweredList.size());
        assertEquals("bbbbbb", loweredList.get(0));
        assertEquals("eeeee", loweredList.get(1));
        assertEquals("cccc", loweredList.get(2));
        assertEquals("aaa", loweredList.get(3));
        assertEquals("a", loweredList.get(4));
        assertEquals("b", loweredList.get(5));
        assertEquals("d", loweredList.get(6));
        assertEquals("d", loweredList.get(7));
    }

    @Test
    public void surroundElements() {
        List<String> list = CollectionsHelper.buildSortByLengthDescandingListFrom("foo", "bar", "baz");
        String before = "[";
        String after = "]";
        List<String> surroundedList = CollectionsHelper.surroundElements(list, before, after);
        assertEquals(3, surroundedList.size());
        assertEquals("[foo]", surroundedList.get(0));
        assertEquals("[bar]", surroundedList.get(1));
        assertEquals("[baz]", surroundedList.get(2));
    }

    @Test
    public void keepOnlyFirstLines() {
        List<String> list = CollectionsHelper.buildListFrom("1", "2", "3", "4", "5", "6", "7", "8",
                "9", "10");
        int numberOfLinesToKeep = 3;
        List<String> shorterList = CollectionsHelper.keepOnlyFirstLines(list, numberOfLinesToKeep);
        assertEquals(3, shorterList.size());
        assertEquals("1", shorterList.get(0));
        assertEquals("2", shorterList.get(1));
        assertEquals("3", shorterList.get(2));
    }

    @Test
    public void appendInfrontOfArray() {
        String firstValue = "eins";
        String[] otherValues = { "zwei", "drei", "vier", "fünf" };
        String[] array = CollectionsHelper.appendInfrontOfArray(firstValue, otherValues);
        assertEquals(5, array.length);
        assertEquals("eins", array[0]);
        assertEquals("zwei", array[1]);
        assertEquals("drei", array[2]);
        assertEquals("vier", array[3]);
        assertEquals("fünf", array[4]);
    }

    @Test
    public void appendAtEndOfArray() {
        String[] otherValues = { "eins", "zwei", "drei", "vier" };
        String lastValue = "fünf";
        String[] array = CollectionsHelper.appendAtEndOfArray(otherValues, lastValue);
        assertEquals(5, array.length);
        assertEquals("eins", array[0]);
        assertEquals("zwei", array[1]);
        assertEquals("drei", array[2]);
        assertEquals("vier", array[3]);
        assertEquals("fünf", array[4]);
    }

    @Test
    public void getSection() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.getSection(list, start, end);
        assertEquals(1, section.size());
        assertEquals("bar", section.get(0));
    }

    @Test
    public void getSectionInLine() {
        List<String> list = CollectionsHelper.buildListFrom("    foo   ", "bar", "    baz    ");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.getSection(list, start, end);
        assertEquals(1, section.size());
        assertEquals("bar", section.get(0));
    }

    @Test (expected = RuntimeException.class)
    public void getSectionNoStartFound() {
        List<String> list = CollectionsHelper.buildListFrom("    foo   ", "bar", "    baz    ");
        String start = "Hugo";
        String end = "baz";
        CollectionsHelper.getSection(list, start, end);
    }

    @Test (expected = RuntimeException.class)
    public void getSectionNoEndFound() {
        List<String> list = CollectionsHelper.buildListFrom("    foo   ", "bar", "    baz    ");
        String start = "foo";
        String end = "Hugo";
        CollectionsHelper.getSection(list, start, end);
    }

    @Test
    public void removeSectionExcludingStartAndEnd() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.removeSectionExcludingStartAndEnd(list, start, end);
        assertEquals(2, section.size());
        assertEquals("foo", section.get(0));
        assertEquals("baz", section.get(1));
    }

    @Test
    public void removeSectionExcludingStartAndEndInLine() {
        List<String> list = CollectionsHelper.buildListFrom("   foo    ", "bar", "     foobazbar");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.removeSectionExcludingStartAndEnd(list, start, end);
        assertEquals(2, section.size());
        assertEquals("   foo    ", section.get(0));
        assertEquals("     foobazbar", section.get(1));
    }

    @Test
    public void removeSectionIncludingStartAndEndRemoveEverything() {
        List<String> list = CollectionsHelper.buildListFrom("foo", "bar", "baz");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.removeSectionIncludingStartAndEnd(list, start, end);
        assertEquals(0, section.size());
    }

    @Test
    public void removeSectionIncludingStartAndEnd() {
        List<String> list = CollectionsHelper.buildListFrom("erste", "foo", "bar", "baz", "letzte");
        String start = "foo";
        String end = "baz";
        List<String> section = CollectionsHelper.removeSectionIncludingStartAndEnd(list, start, end);
        assertEquals(2, section.size());
        assertEquals("erste", section.get(0));
        assertEquals("letzte", section.get(1));
    }

    @Test
    public void anyLineContains() {
        List<String> list = CollectionsHelper.buildListFrom("  foo   ", "  bar ", " baz  ");
        assertTrue(CollectionsHelper.anyLineContains(list, "foo"));
        assertTrue(CollectionsHelper.anyLineContains(list, "bar"));
        assertTrue(CollectionsHelper.anyLineContains(list, "baz"));
        assertFalse(CollectionsHelper.anyLineContains(list, "foobar"));
    }

    @Test
    public void findAllIndces() {
        List<String> list = CollectionsHelper.buildListFrom(
                "foo", "bar", "foo", "baz", "foo", "nichtfoo");
        List<Integer> indices = CollectionsHelper.findAllIndces("foo", list);
        assertEquals(3, indices.size());
        assertEquals(Integer.valueOf(0), indices.get(0));
        assertEquals(Integer.valueOf(2), indices.get(1));
        assertEquals(Integer.valueOf(4), indices.get(2));
    }

    @Test
    public void findAllIndcesInt() {
        List<Integer> list = CollectionsHelper.buildListFrom(1, 2, 1, 3, 1, 4);
        List<Integer> indices = CollectionsHelper.findAllIndces(1, list);
        assertEquals(3, indices.size());
        assertEquals(Integer.valueOf(0), indices.get(0));
        assertEquals(Integer.valueOf(2), indices.get(1));
        assertEquals(Integer.valueOf(4), indices.get(2));
    }

    @Test
    public void joinStringLists() {
        List<String> list1 = CollectionsHelper.buildListFrom(
                "foo", "bar", "baz");
        List<String> list2 = CollectionsHelper.buildListFrom(
                "Äpfel", "Birnen", "Zitronen");
        List<String> list = CollectionsHelper.joinStringLists(list1, list2);
        assertEquals(6, list.size());
        assertEquals(list1.get(0), list.get(0));
        assertEquals(list1.get(1), list.get(1));
        assertEquals(list1.get(2), list.get(2));
        assertEquals(list2.get(0), list.get(3));
        assertEquals(list2.get(1), list.get(4));
        assertEquals(list2.get(2), list.get(5));
    }

    @Test
    public void copyList() {
        List<String> list = CollectionsHelper.buildListFrom("Äpfel", "Birnen", "Zitronen");
        List<String> copied = CollectionsHelper.copyList(list);
        list = CollectionsHelper.buildListFrom("Automaten", "Bastionen", "Zentralen");
        assertEquals(CollectionsHelper.buildListFrom("Äpfel", "Birnen", "Zitronen"), copied);
    }

    @Test
    public void trimList() {
        List<String> list = CollectionsHelper.buildListFrom("  Äpfel ", "Birnen    ", "Zitronen");
        List<String> trimmed = CollectionsHelper.trimList(list);
        assertEquals(CollectionsHelper.buildListFrom("Äpfel", "Birnen", "Zitronen"), trimmed);
    }
    
}
