package de.duehl.basics.io.textfile;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.junit.Test;

import de.duehl.basics.io.Charset;
import de.duehl.basics.io.FileHelper;

public class StringsFromNormalTextFileReaderTest {

    private final String path;

    private String path() {
        String className = getClass().getName();
        int dotIndex = className.indexOf(".");
        while (dotIndex > -1) {
            className = className.substring(dotIndex + 1); // endet nicht auf .
            dotIndex = className.indexOf(".");
        }

        URL url = getClass().getResource(className + ".class");
        File f = new File(url.getPath());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        String p = f.getPath() + File.separator + "data" + File.separator
                + "testfiles" + File.separator;
        return p;
    }

    public StringsFromNormalTextFileReaderTest() {
        path = path();
    }


    /* -------------------------- TESTS -------------------------- */


    @Test
    public void readStandard() {
        StringsFromTextFileReader reader = new StringsFromNormalTextFileReader(
                FileHelper.concatPathes(path, "test_datei.txt"), Charset.UTF_8, "#");
        reader.beQuiet();
        List<String> list = reader.read();
        assertEquals(4, list.size());
        assertEquals("eins", list.get(0));
        assertEquals("zwei", list.get(1));
        assertEquals("drei", list.get(2));
        assertEquals("GroßBuchstAben?", list.get(3));
    }

    @Test
    public void readNoTrim() {
        StringsFromTextFileReader reader = new StringsFromNormalTextFileReader(
                FileHelper.concatPathes(path, "test_datei.txt"), Charset.UTF_8, "#");
        reader.beQuiet();
        reader.doNotTrimLines();
        List<String> list = reader.read();
        assertEquals(4, list.size());
        assertEquals("eins", list.get(0));
        assertEquals("    zwei", list.get(1));
        assertEquals("  drei       ", list.get(2));
        assertEquals("GroßBuchstAben?", list.get(3));
    }

    @Test
    public void readToLowerCase() {
        StringsFromTextFileReader reader = new StringsFromNormalTextFileReader(
                FileHelper.concatPathes(path, "test_datei.txt"), Charset.UTF_8, "#");
        reader.beQuiet();
        reader.changeToLowerCase();
        List<String> list = reader.read();
        assertEquals(4, list.size());
        assertEquals("eins", list.get(0));
        assertEquals("zwei", list.get(1));
        assertEquals("drei", list.get(2));
        assertEquals("großbuchstaben?", list.get(3));
    }

    @Test
    public void readIncludeBlankLines() {
        StringsFromTextFileReader reader = new StringsFromNormalTextFileReader(
                FileHelper.concatPathes(path, "test_datei.txt"), Charset.UTF_8, "#");
        reader.beQuiet();
        reader.includeBlankLinesIntoResult();
        List<String> list = reader.read();
        assertEquals(11, list.size());
        assertEquals("", list.get(0));
        assertEquals("eins", list.get(1));
        assertEquals("", list.get(2));
        assertEquals("", list.get(3));
        assertEquals("zwei", list.get(4));
        assertEquals("", list.get(5));
        assertEquals("drei", list.get(6));
        assertEquals("", list.get(7));
        assertEquals("", list.get(8));
        assertEquals("GroßBuchstAben?", list.get(9));
        assertEquals("", list.get(10));
    }

}
