package de.duehl.swing.ui.buttons.painted;

import java.awt.Color;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.swing.ui.PixelPoint;
import de.duehl.swing.ui.buttons.ButtonHelper;

/**
 * Diese Klasse stellt einen kleinen Schalter zum Starten von etwas dar.
 *
 * @version 1.01     2017-12-21
 * @author Christian Dühl
 */

public class GoButton extends PaintedButton {

    private static final long serialVersionUID = -1441507523499749679L;

    /** Farbe des Pfeils. */
    private final Color goColor;

    /** Konstruktor. */
    public GoButton() {
        this(Color.BLACK);
    }

    /**
     * Konstruktor.
     *
     * @param goColor
     *            Farbe des Pfeils.
     */
    public GoButton(Color goColor) {
        super();
        this.goColor = goColor;
    }

    /**
     * Farbwahl abhängig davon, ob die Maus über dem Button ist.
     *
     * Die gewählte Farbe normalerweise, wenn die Maus darüber kommt, die
     * Komplementärfarbe.
     */
    @Override
    protected void setSymbolColor() {
        graphics2.setColor(goColor);
        if (getModel().isRollover()) {
            graphics2.setColor(ButtonHelper.antiColor(goColor));
        }
    }

    /** Zeichnet das Symbol in horizontaler und rechteckiger Anordnung. */
    @Override
    protected void paintHorizontalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und rechteckiger Anordnung. */
    @Override
    protected void paintVerticalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in horizontaler und quadratischer Anordnung. */
    @Override
    protected void paintHorizontalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und quadratischer Anordnung. */
    @Override
    protected void paintVerticalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet den kleinen Pfeil zum Starten. */
    private void paintSymbol() {
        int width = getWidth();
        int height = getHeight();

        if (isForceSquare()) {
            width = Math.min(width, height);
            height = width;
        }

        int delta = 3;

        int firstLeftPixel = (getWidth() - width)/2;
        int left = firstLeftPixel + delta;

        int lastRightPixel = firstLeftPixel + width - 1;
        int right = lastRightPixel - delta;

        int firstUpperPixel = (getHeight() - height)/2;
        int upper = firstUpperPixel + delta;

        int lastLowerPixel = firstUpperPixel + height - 1;
        int lower = lastLowerPixel - delta;

        int midHeight = firstUpperPixel + height / 2;
        int midWidth = firstLeftPixel + width / 2;

        if (isHorizontal()) {
            paintPolygon(
                    new PixelPoint(left,   upper    ),
                    new PixelPoint(right,  midHeight),
                    new PixelPoint(left,   lower    )
                    );
        }
        else {
            paintPolygon(
                    new PixelPoint(left,     upper),
                    new PixelPoint(midWidth, lower),
                    new PixelPoint(right,    upper)
                    );
        }
    }

}