package de.duehl.swing.ui.colors;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Color;

/**
 * Diese Klasse stellt grundlegende, einfache Methoden rund um Farben zur
 * Verfügung.
 *
 * @version 1.01     2013-12-12
 * @author Christian Dühl
 */

public class ColorTool {

    /**
     * Bildet aus einem der für die Anzeige als HTML-Datei gebildeten Farbnamen
     * der Form "FF0000" ein passendes Color-Objekt.
     *
     * @param hexColor
     *            Farbe in der Art "FF0000".
     * @return Color-Objekt mit der entsprechenden Farbe (im Beispiel
     *         Color.RED).
     * @throws UnknownColorException
     *             Wenn die Farbe nicht 6 Zeichen lang ist oder nicht zur Basis
     *             16 in einen Integer geparst werden kann.
     */
    public Color hexColorToJavaColor(String hexColor) {
        return ColorTranslator.hex2Swing(hexColor);
    }

    /**
     * Bildet aus einer Java-Farbe einen HTML-Farbwert in der Form "FF0000".
     *
     * @param color
     *            Java-Farbe (etwa Color.RED).
     * @return HTML-Farbwert (etwa "FF0000").
     */
    public String javaColorToHexColor(Color color) {
        return ColorTranslator.swing2Hex(color);
    }

    /** Berechnet die entgegengesetzte Farbe. */
    public Color anticolor(Color color) {
        int red   = 255 - color.getRed();
        int green = 255 - color.getGreen();
        int blue  = 255 - color.getBlue();
        return new Color(red, green, blue);
    }

    /**
     * Berechnet die entgegengesetzte Farbe.
     *
     * @param hexColor
     *            Farbe in der Art "FF0000".
     * @throws UnknownColorException
     *             Wenn die Farbe nicht 6 Zeichen lang ist oder nicht zur Basis
     *             16 in einen Integer geparst werden kann.
     */
    public String anticolor(String hexColor) {
        Color color = hexColorToJavaColor(hexColor);
        Color antiColor = anticolor(color);
        String antiHexColor = javaColorToHexColor(antiColor);
        return antiHexColor;
    }

}
