package de.duehl.swing.ui.key;

import static de.duehl.swing.ui.key.KeybingingDefinitions.*;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

/**
 * Diese Klasse belegt eine RootPane mit bestimmten Tastenkombinationen.
 *
 * @version 1.01     2018-02-08
 * @author Christian Dühl
 */

public class BindKeysOnRootPane {

    private final JRootPane rootPane;

    public BindKeysOnRootPane(JRootPane rootPane) {
        this.rootPane = rootPane;
    }

    /** Fügt die Tastaturbindung Escape hinzu. */
    public void setKeyBindingEscape(Runnable escapeRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_ESCAPE, KEYBINDING_ESCAPE);
        putIntoActionMapOfRootPane(KEYBINDING_ESCAPE, runnableToAction(escapeRunnable));
    }

    /** Fügt die Tastaturbindung Home (Pos1) hinzu. */
    public void setKeyBindingHome(Runnable homeRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_POS1, KEYBINDING_POS1);
        putIntoActionMapOfRootPane(KEYBINDING_POS1, runnableToAction(homeRunnable));
    }

    /** Fügt die Tastaturbindung End (Ende) hinzu. */
    public void setKeyBindingEnd(Runnable endRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_END, KEYBINDING_END);
        putIntoActionMapOfRootPane(KEYBINDING_END, runnableToAction(endRunnable));
    }

    /** Fügt die Tastaturbindung Home (Pos1) hinzu. */
    public void setKeyBindingCtrlHome(Runnable ctrlHomeRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_CTRL_POS1, KEYBINDING_CTRL_POS1);
        putIntoActionMapOfRootPane(KEYBINDING_CTRL_POS1, runnableToAction(ctrlHomeRunnable));
    }

    /** Fügt die Tastaturbindung End (Ende) hinzu. */
    public void setKeyBindingCtrlEnd(Runnable ctrlEndRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_CTRL_END, KEYBINDING_CTRL_END);
        putIntoActionMapOfRootPane(KEYBINDING_CTRL_END, runnableToAction(ctrlEndRunnable));
    }

    /** Fügt die Tastaturbindung PageUp (Bild auf) hinzu. */
    public void setKeyBindingPageUp(Runnable pageUpRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_PAGE_UP, KEYBINDING_PAGE_UP);
        putIntoActionMapOfRootPane(KEYBINDING_PAGE_UP, runnableToAction(pageUpRunnable));
    }

    /** Fügt die Tastaturbindung PageDown (Bild runter) hinzu. */
    public void setKeyBindingPageDown(Runnable pageDownRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_PAGE_DOWN, KEYBINDING_PAGE_DOWN);
        putIntoActionMapOfRootPane(KEYBINDING_PAGE_DOWN, runnableToAction(pageDownRunnable));
    }

    /** Fügt die Tastaturbindung Enter hinzu. */
    public void setKeyBindingEnter(Runnable enterRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_ENTER, KEYBINDING_ENTER);
        putIntoActionMapOfRootPane(KEYBINDING_ENTER, runnableToAction(enterRunnable));
    }

    /** Fügt die Tastaturbindung Ctrl-Enter hinzu. */
    public void setKeyBindingCtrlEnter(Runnable ctrlEnterRunnable) {
        putIntoInputMapWhenInFocusedWindow(KEYSTROKE_CTRL_ENTER, KEYBINDING_CTRL_ENTER);
        putIntoActionMapOfRootPane(KEYBINDING_CTRL_ENTER, runnableToAction(ctrlEnterRunnable));
    }

    private void putIntoInputMapWhenInFocusedWindow(KeyStroke keyStroke, String actrionMapKey) {
        InputMap inputMap = getInputMapWhenInFocusedWindow();
        inputMap.put(keyStroke, actrionMapKey);
    }

    private InputMap getInputMapWhenInFocusedWindow() {
        InputMap inputMap = rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        return inputMap;
    }

    private void putIntoActionMapOfRootPane(String keybinding, Action action) {
        rootPane.getActionMap().put(keybinding, action);
    }

    private AbstractAction runnableToAction(Runnable runnable) {
        return new AbstractAction() {
            private static final long serialVersionUID = 1L;

            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                runnable.run();
            }
        };
    }

}
