package de.duehl.swing.ui.text;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.NonModalFrameDialogBase;
import de.duehl.swing.ui.text.data.TextWorker;

/**
 * Diese Klasse zeigt etwas in einem Textfenster an, das bearbeitet und dessen Ergebnis wieder
 * angezeigt werden kann.
 *
 * @version 1.01     2019-07-02
 * @author Christian Dühl
 */

public class WorkOnTextGui extends NonModalFrameDialogBase {

    private static final Dimension CHECKER_DIMENSION = new Dimension(800, 650);

    /** Objekt dass nach Klick auf den Button den ggf. veränderten Text übergeben bekommt. */
    private final TextWorker textWorker;

    /** Gui-Element mit dem Text. */
    private final JEditorPane editor;

    /** Gui-Element mit dem Button. */
    private final JButton runButton;

    /** Konstruktor für die Bearbeitung von Texten. */
    public WorkOnTextGui(TextWorker textWorker) {
        super(null, "PLEASE SET YOUR TITLE WITH gui.setTitle(...)", CHECKER_DIMENSION);
        setLocation(300, 200);
        addClosingWindowListener(() -> closeDialog());

        this.textWorker = textWorker;

        editor = new JEditorPane();
        runButton = new JButton();

        fillDialog();
    }

    @Override
    protected void populateDialog() {
        add(createTextPart(), BorderLayout.CENTER);
        add(createButtonPart(), BorderLayout.SOUTH);
    }

    private Component createTextPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        //editor.setText(createTitle());
        editor.setCaretPosition(0);
        //editor.setContentType("text/html");
        GuiTools.setMonospacedFont(editor);

        JScrollPane scroll = new JScrollPane(editor);
        scroll.setPreferredSize(CHECKER_DIMENSION);

        panel.add(scroll, BorderLayout.CENTER);

        return panel;
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.EAST);
        panel.add(createCorrectButton(), BorderLayout.CENTER);

        return panel;
    }

    private Component createCorrectButton() {
        setButtonText("PLEASE SET YOUR RUN BUTTON TEXT WITH gui.setButtonText(...)");
        runButton.addActionListener(e -> workOnInput());
        return runButton;
    }

    public void setButtonText(String buttonText) {
        runButton.setText(buttonText);
    }

    private void workOnInput() {
        String contents = editor.getText();

        startLongTimeProcess("Verarbeitung läuft");
        new Thread(() -> workOnInputInThread(contents)).start();
    }

    private void workOnInputInThread(String contents) {
        String correctedContents = textWorker.workOnText(contents);
        SwingUtilities.invokeLater(() -> showTextInEdt(correctedContents));
    }

    private void showTextInEdt(String correctedContents) {
        editor.setText(correctedContents);
        editor.validate();
        endLongTimeProcess();
    }

    private JButton createQuitButton() {
        JButton button = new JButton("Beenden");
        button.addActionListener(e -> closeDialog());
        return button;
    }

}
